@@ -1,14 +1,14 @@
 /*
- ** Copyright 2004-2008 - INL
+ ** Copyright 2004-2009 - INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: libnuclient.c 5466 2008-12-30 15:55:50Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,32 +37,24 @@
  */
 #define USE_GCRYPT_MALLOC_SECURE
 
-#include "nufw_source.h"
+#include "libnuclient.h"
 #include "nuclient.h"
-#include <pthread.h>
+#include "nufw_source.h"
+#include "nuclient_plugins.h"
+
 #include <sasl/sasl.h>
 #include <sasl/saslutil.h>
 #include <stdarg.h>		/* va_list, va_start, ... */
-#include <gnutls/x509.h>
-#include <gcrypt.h>
 #include <langinfo.h>
 #include <proto.h>
-#include "client.h"
 #include "security.h"
 #include "sys_config.h"
 #include "internal.h"
+#include "tcptable.h"
 #include <sys/utsname.h>
 
-#ifndef GCRY_THREAD
-#define GCRY_THREAD 1
-GCRY_THREAD_OPTION_PTHREAD_IMPL;
-#endif
-
-#ifndef NUCLIENT_WITHOUT_DIFFIE_HELLMAN
-#  define DH_BITS 1024
-#endif
-
-static const int cert_type_priority[3] = { GNUTLS_CRT_X509, 0 };
+#include <nussl.h>
+#include <nubase.h>
 
 
 void nu_exit_clean(nuauth_session_t * session)
@@ -70,25 +62,15 @@ void nu_exit_clean(nuauth_session_t * session)
 	if (session->ct) {
 		tcptable_free(session->ct);
 	}
-	if (session->socket > 0) {
-		shutdown(session->socket, SHUT_WR);
-		close(session->socket);
-		session->socket = 0;
+
+	if (session->nussl) {
+		nussl_session_destroy(session->nussl);
+		session->nussl = NULL;
 	}
 
 	secure_str_free(session->username);
 	secure_str_free(session->password);
-	free(session->ca_file);
 
-	if (session->cred) {
-		gnutls_certificate_free_keys(session->cred);
-		gnutls_certificate_free_credentials(session->cred);
-	}
-	gnutls_deinit(session->tls);
-
-	pthread_cond_destroy(&(session->check_cond));
-	pthread_mutex_destroy(&(session->check_count_mutex));
-	pthread_mutex_destroy(&(session->mutex));
 	free(session);
 }
 
@@ -101,7 +83,8 @@ void nu_exit_clean(nuauth_session_t * session)
  *  - nu_client_new() or nu_client_new_callback(): start user session
  *  - nu_client_setup_tls(): (optionnal) setup TLS key/certificate files
  *  - nu_client_connect(): try to connect to nuauth server
- *  - nu_client_check(): do a check, it has to be run at regular interval
+ *  - nu_client_check(): check if there is packet to authenticate and send authentication
+ *  request to nuauth if needed. It has to be run in a endless loop.
  *  - nu_client_delete(): free a user session
  *  - nu_client_global_deinit(): To be called once at program end
  *
@@ -118,9 +101,7 @@ void nu_exit_clean(nuauth_session_t * session)
  */
 void nu_client_delete(nuauth_session_t * session)
 {
-	/* kill all threads */
 	ask_session_end(session);
-	/* all threads are dead, we are the one who can access to it */
 	/* destroy session */
 	nu_exit_clean(session);
 }
@@ -138,11 +119,11 @@ void nu_client_delete(nuauth_session_t * session)
 int nu_client_global_init(nuclient_error_t * err)
 {
 	int ret;
+	prg_cache_init();
 
-	gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-	ret = gnutls_global_init();
-	if (ret != 0) {
-		SET_ERROR(err, GNUTLS_ERROR, ret);
+	if (nussl_init() != NUSSL_OK)
+	{
+		SET_ERROR(err, INTERNAL_ERROR, NUSSL_INIT_ERR); /* TODO: patch nussl to handle errors correctly in nussl_sock_init */
 		return 0;
 	}
 
@@ -155,16 +136,54 @@ int nu_client_global_init(nuclient_error_t * err)
 	/* get local charset */
 	nu_locale_charset = nl_langinfo(CODESET);
 	if (nu_locale_charset == NULL) {
-		nu_error_log(err, "Can't get locale charset!");
-		SET_ERROR(err, INTERNAL_ERROR, UNKNOWN_ERR);
+		fprintf(stderr, "Can't get locale charset!\n");
 		return 0;
 	}
 
+	/* init capabilities string */
+	ret = snprintf(nu_capabilities, NU_CAPABILITIES_MAXLENGTH, "%s", NU_CAPABILITIES);
+	if (ret <= 0) {
+		return 0;
+	}
+
+	INIT_LLIST_HEAD(&nu_postauth_extproto_l);
+	INIT_LLIST_HEAD(&nu_cruise_extproto_l);
+
+	init_plugins();
+
+	return 1;
+}
+
+/**
+ * \ingroup nuclientAPI
+ * \brief Initialization: load config file
+ *
+ * This function loads the config file, and must be called after nu_client_global_init()
+ *
+ * \warning To be called only once.
+ */
+int nu_client_init_config()
+{
 	load_sys_config();
 
 	return 1;
 }
 
+/**
+ * \ingroup nuclientAPI
+ * \brief Initialization: load plugins
+ *
+ * This function loads the plugins, and must be called after nu_client_global_init() and nu_client_init_config
+ *
+ * \warning To be called only once.
+ */
+int nu_client_init_plugins()
+{
+	load_plugins();
+
+	return 1;
+}
+
 /**
  * \ingroup nuclientAPI
  * \brief  Global de init function
@@ -174,8 +193,6 @@ int nu_client_global_init(nuclient_error_t * err)
 void nu_client_global_deinit()
 {
 	sasl_done();
-	gnutls_global_deinit();
-	free_sys_config();
 }
 
 /**
@@ -221,7 +238,7 @@ char *nu_get_home_dir()
 
 	uid = getuid();
 	if (!(pwd = getpwuid(uid))) {
-		printf("Unable to get password file record\n");
+		log_printf(DEBUG_LEVEL_CRITICAL, "Unable to get password file record");
 		endpwent();
 		return NULL;
 	}
@@ -245,7 +262,7 @@ char *nu_get_user_name()
 
 	uid = getuid();
 	if (!(pwd = getpwuid(uid))) {
-		printf("Unable to get password file record\n");
+		log_printf(DEBUG_LEVEL_CRITICAL,"Unable to get password file record");
 		endpwent();
 		return NULL;
 	}
@@ -254,284 +271,340 @@ char *nu_get_user_name()
 	return name;
 }
 
-int check_key_perms(const char* filename)
-{
-	struct stat infos;
+/**
+ * \ingroup nuclientAPI
+ * Add capability to the list of supported capabilities
+ *
+ * \return 0 if ok, < 0 if not
+ */
 
-	if (stat(filename, &infos) != 0) {
-		return 0;
+static int _nu_client_set_capability(char *pcapa, const char *capa)
+{
+	strncat(pcapa, ";", NU_CAPABILITIES_MAXLENGTH - strlen(pcapa));
+	strncat(pcapa, capa, NU_CAPABILITIES_MAXLENGTH - strlen(pcapa));
+
+	return 0;
+}
+
+/**
+ * \ingroup nuclientAPI
+ * Remove capability from the list of supported capabilities
+ *
+ * \return 0 if ok, < 0 if not
+ */
+
+static int _nu_client_unset_capability(char *pcapa, const char *capa)
+{
+	char * start, * end;
+	start = strstr(pcapa, capa);
+	if (start == NULL) {
+		return -ENOSTR;
+	}
+	end = strstr(start, ";");
+	*(start - 1) = '\0';
+	if (end != NULL) {
+		strcat(pcapa, end);
+	}
+	return 0;
+}
+
+int nu_client_set_capability(const char *capa)
+{
+	return _nu_client_set_capability(nu_capabilities, capa);
+}
+
+int nu_client_unset_capability(const char *capa)
+{
+	return _nu_client_unset_capability(nu_capabilities, capa);
+}
+
+int nu_client_set_session_capability(nuauth_session_t *session, const char *capa)
+{
+	if (session->nu_capabilities[0] == 0) {
+		strcpy(session->nu_capabilities, nu_capabilities);
+	}
+	return _nu_client_set_capability(session->nu_capabilities, capa);
+}
+
+int nu_client_unset_session_capability(nuauth_session_t *session, const char *capa)
+{
+	return _nu_client_unset_capability(session->nu_capabilities, capa);
+}
+
+
+
+void nu_client_set_client_info(nuauth_session_t *session,
+		const char *client_name, const char *client_version)
+{
+	if (client_name)
+		session->client_name = strdup(client_name);
+
+	if (client_version)
+		session->client_version = strdup(client_version);
+}
+
+int nu_client_set_key(nuauth_session_t* session, const char* keyfile, const char* certfile, nuclient_error_t* err)
+{
+	if (session->pem_key)
+		free(session->pem_key);
+
+	if (session->pem_cert)
+		free(session->pem_cert);
+
+	if (keyfile)
+	{
+		session->pem_key = strdup(keyfile);
+		log_printf(DEBUG_LEVEL_DEBUG, "Using key: %s", keyfile);
 	}
 
-	/* File should not be readable or writable by others */
-	if (infos.st_mode & S_IROTH || infos.st_mode & S_IWOTH) {
-		return 0;
+	if (certfile)
+	{
+		session->pem_cert = strdup(certfile);
+		log_printf(DEBUG_LEVEL_DEBUG, "Using certificate: %s", certfile);
 	}
 
 	return 1;
 }
 
-static int _cb_request_cert(gnutls_session_t session, const gnutls_datum_t* req_ca_dn, int nreqs, const gnutls_pk_algorithm_t* pk_algos, int pk_algos_length, gnutls_retr_st* st)
+int nu_client_set_ca(nuauth_session_t* session, const char* cafile, nuclient_error_t* err)
 {
-	printf("TLS error: server requests certificate, none configured\n");
-	return 0;
+	if (session->pem_ca)
+		free(session->pem_ca);
+
+	if (cafile)
+		session->pem_ca = strdup(cafile);
+
+	log_printf(DEBUG_LEVEL_DEBUG, "Using CA: %s", cafile);
+	return 1;
 }
 
-/* read the file indicated (by na1me) in the fname parameter.  store
-   its entire contents in a single datum. */
-int set_datum_file(gnutls_datum_t* d, const char* fname) {
-	struct stat sbuf;
-	unsigned char* c = NULL;
-	FILE* file = NULL;
-	size_t x = 0;
-
-	if (0 != stat(fname, &sbuf)) {
-		fprintf(stderr, "Could not access file '%s'\n", fname);
-		return -1;
-	}
-
-	c = gnutls_realloc(d->data, sbuf.st_size);
-	if (NULL == c) {
-		fprintf(stderr, "failed to allocate %ld bytes for '%s'\n", (long)sbuf.st_size, fname);
-		return -1;
-	}
-
-	d->data = c;
-	d->size = sbuf.st_size;
-
-	file = fopen(fname, "r");
-	if (NULL == file) {
-		fprintf(stderr, "failed to open '%s' for reading\n",  fname);
-		return -1;
-	}
-
-	x = fread(d->data, d->size, 1, file);
-	if (x != 1) {
-		fprintf(stderr, "tried to read %d bytes, read %zd instead from '%s'\n", d->size, x, fname);
-		fclose(file);
-		return -1;
-	}
-
-	fclose(file);
-	return 0;
-}
-
-
-/* return -1 in case of error or expiration time if operation is successful */
-int check_crl_validity(const char *crl_file, const char *ca_file,
-			      nuclient_error_t *err)
+int nu_client_set_pkcs12(nuauth_session_t* session, char* key_file, char* key_password, nuclient_error_t* err)
 {
-	gnutls_x509_crt_t ca;
-	gnutls_x509_crl_t crl;
-	gnutls_datum_t datum_crl;
-	gnutls_datum_t datum_ca;
-	time_t t, now;
+	if (session->pkcs12_file)
+		free(session->pkcs12_file);
+
+	if (session->pkcs12_password)
+		free(session->pkcs12_password);
+
+	if (key_file)
+	{
+		log_printf(DEBUG_LEVEL_DEBUG, "Using key: %s", key_file);
+		session->pkcs12_file = strdup(key_file);
+	}
+
+	if (key_password)
+		session->pkcs12_password = strdup(key_password);
+
+	return 1;
+}
+/**
+ * \ingroup nuclientAPI
+ * Initialize TLS:
+ *    - Set key filename (and test if the file does exist)
+ *    - Set certificate (if key and cert. are present)
+ *
+ * \param session Pointer to client session
+ * \param keyfile Complete path to a key file stored in PEM format (can be NULL)
+ * \param certfile Complete path to a certificate file stored in PEM format (can be NULL)
+ * \param err Pointer to a nuclient_error_t: which contains the error
+ * \return Returns 0 on error (error description in err), 1 otherwise
+ */
+int nu_client_load_key(nuauth_session_t * session,
+			const char *keyfile, const char *certfile,
+			nuclient_error_t * err)
+{
+	char certstring[256];
+	char keystring[256];
+	char *home = nu_get_home_dir();
+	int exit_on_error = 0;
 	int ret;
 
-	if (!ca_file || !crl_file)
-		return -1;
+	/* If the user specified a certficate and a key on command line,
+	 * exit if we fail loading them.
+	 * Elsewise, try loading certs from ~/.nufw/, but continue if we fail
+	 */
+	if (certfile || keyfile)
+		exit_on_error = 1;
 
-	datum_ca.data = NULL;
-	datum_crl.data = NULL;
-
-	ret = set_datum_file(&datum_crl, crl_file);
-	if (ret) {
-	        nu_error_log(err, "TLS: Could not read CRL file \"%s\"", crl_file);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		return -1;
-	}
-	ret = set_datum_file(&datum_ca, ca_file);
-	if (ret) {
-	        nu_error_log(err, "TLS: Could not read CA file \"%s\"", ca_file);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		gnutls_free(datum_crl.data);
-		return -1;
+	/* compute patch keyfile */
+	if (keyfile == NULL && home != NULL) {
+		ret = secure_snprintf(keystring, sizeof(keystring),
+				     "%s/.nufw/key.pem", home);
+		if (ret)
+			keyfile = keystring;
 	}
 
-	gnutls_x509_crt_init(&ca);
-	gnutls_x509_crl_init(&crl);
-
-	ret = gnutls_x509_crl_import(crl, &datum_crl, GNUTLS_X509_FMT_PEM);
-	if (ret) {
-	        nu_error_log(err, "TLS: Could not import CRL data from \"%s\"",
-			     crl_file);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		gnutls_free(datum_ca.data);
-		gnutls_free(datum_crl.data);
-		return -1;
+	if (certfile == NULL && home != NULL) {
+		ret = secure_snprintf(certstring, sizeof(certstring),
+				     "%s/.nufw/cert.pem", home);
+		if (ret)
+			certfile = certstring;
 	}
 
-	ret = gnutls_x509_crt_import(ca, &datum_ca, GNUTLS_X509_FMT_PEM);
-	if (ret) {
-	        nu_error_log(err, "TLS: Could not import CA data from \"%s\"",
-			     crl_file);
-		SET_ERROR(err, GNUTLS_ERROR, ret);
-		gnutls_free(datum_ca.data);
-		gnutls_free(datum_crl.data);
-		return -1;
-	}
+	if (certfile != NULL || keyfile != NULL) {
+		ret = nussl_ssl_set_keypair(session->nussl, certfile, keyfile);
 
-
-	/* Check if CRL was signed by configured CA */
-	ret = gnutls_x509_crl_check_issuer (crl, ca);
-	if (ret != 1) {
-	        nu_error_log(err, "TLS: CRL issuer is NOT the configured certificate authority for \"%s\"",
-			     crl_file);
-		SET_ERROR(err, GNUTLS_ERROR, ret);
-		return -1;
-	}
-
-	/* Check if CRL has expired */
-	now = time(NULL);
-	t = gnutls_x509_crl_get_next_update (crl);
-	/* This field is optional in a CRL */
-	if (t != (time_t)-1 ) {
-		if (now > t) {
-			nu_error_log(err, "TLS: CRL \"%s\" has expired and should be re-issued."
-				          " TLS session will not be established.",
-				     crl_file);
-			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_EXPIRED);
-			/* this is only a warning, do not decrement return_value */
-			return -1;
+		if (ret != NUSSL_OK) {
+			if (exit_on_error) {
+				if (home)
+					free(home);
+				SET_ERROR(err, NUSSL_ERR, ret);
+				return 0;
+			}
+			else {
+				log_printf(DEBUG_LEVEL_WARNING, "Warning: Failed to load default certificate and key.");
+			}
 		}
 	}
 
-	gnutls_x509_crt_deinit(ca);
-	gnutls_x509_crl_deinit(crl);
-	gnutls_free(datum_ca.data);
-	gnutls_free(datum_crl.data);
+	if (home)
+		free(home);
 
-	return t;
+	return 1;
 }
 
 
 /**
  * \ingroup nuclientAPI
  * Initialize TLS:
- *    - Set key filename (and test if the file does exist)
- *    - Set certificate filename (and test if the file does exist)
- *    - Set trust file of credentials (if needed)
- *    - Set certificate (if key and cert. are present)
- *    - Init. TLS session
+ *    - Set PKCS12 key/certificate filename (and test if the file does exist)
+ *    - Set PKCS12 password
  *
  * \param session Pointer to client session
- * \param user_keyfile Complete path to a key file stored in PEM format (can be NULL)
- * \param user_certfile Complete path to a certificate file stored in PEM format (can be NULL)
- * \param user_cafile Complete path to a certificate authority file stored in PEM format (can be NULL)
- * \param tls_password Certificate password string
+ * \param pkcs12file Complete path to a key and a certificate file stored in PEM format (can be NULL)
+ * \param pkcs12password Password of the pkcs12 file
  * \param err Pointer to a nuclient_error_t: which contains the error
  * \return Returns 0 on error (error description in err), 1 otherwise
  */
-int nu_client_setup_tls(nuauth_session_t * session,
-			char *user_keyfile, char *user_certfile, char *user_cafile,
-			char *tls_password, nuclient_error_t * err)
+int nu_client_load_pkcs12(nuauth_session_t * session,
+			char *pkcs12file, char *pkcs12password,
+			nuclient_error_t * err)
 {
-	const char *keyfile = NULL;
-	const char *certfile = NULL;
-	const char *cafile = NULL;
-	int ret;
-
-	session->tls_password = tls_password;
-
-	if (user_cafile) {
-		cafile = user_cafile;
-	} else {
-		cafile = nu_client_default_tls_ca();
-	}
-
-	/* test if ca exists */
-	if (cafile != NULL && access(cafile, R_OK) != 0) {
-		nu_error_log(err, "Unable to read CA file \"%s\"", cafile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		errno = EBADF;
+	int ret = nussl_ssl_set_pkcs12_keypair(session->nussl, pkcs12file, pkcs12password);
+	if (ret != NUSSL_OK)
+	{
+		SET_ERROR(err, NUSSL_ERR, ret);
 		return 0;
 	}
-
-	/* sets the trusted cas file */
-	if (cafile != NULL) {
-		ret = gnutls_certificate_set_x509_trust_file(session->cred, cafile,
-							   GNUTLS_X509_FMT_PEM);
-		if (ret < 0) {
-			SET_ERROR(err, GNUTLS_ERROR, ret);
-			return 0;
-		}
-		session->need_ca_verif = 1;
-		session->ca_file = strdup(cafile);
-	} else {
-		session->ca_file = NULL;
-	}
-
-	/* compute patch keyfile */
-	if (user_keyfile) {
-		keyfile = user_keyfile;
-	} else {
-		keyfile = nu_client_default_tls_key();
-	}
-
-	/* test if key file exists */
-	if (keyfile != NULL && access(keyfile, R_OK) != 0) {
-		nu_error_log(err, "Unable to read key file \"%s\"", keyfile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		errno = EBADF;
-		return 0;
-	}
-
-	if (user_certfile) {
-		certfile = user_certfile;
-	} else {
-		certfile = nu_client_default_tls_cert();
-	}
-
-	/* test if cert exists */
-	if (certfile != NULL && access(certfile, R_OK) != 0) {
-		nu_error_log(err, "Unable to read certificate file \"%s\"", certfile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		errno = EBADF;
-		return 0;
-	}
-
-	if (keyfile && !check_key_perms(keyfile)) {
-		nu_error_log(err, "Permissions on private key \"%s\" are not restrictive enough, file should not be readable or writable by others.",
-			     keyfile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		errno = EPERM;
-		return 0;
-	}
-
-	if (certfile != NULL && keyfile != NULL) {
-		ret = gnutls_certificate_set_x509_key_file(session->cred,
-							 certfile, keyfile,
-							 GNUTLS_X509_FMT_PEM);
-		if (ret < 0) {
-			nu_error_log(err, "Can't use x509 files (%s)", gnutls_strerror(ret));
-			SET_ERROR(err, GNUTLS_ERROR, ret);
-			return 0;
-		}
-	} else {
-		if (keyfile != NULL) {
-			nu_error_log(err, "key file \"%s\" is set but no cert"
-					" file is.", keyfile);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			return 0;
-		} else if (certfile != NULL) {
-			nu_error_log(err, "cert file \"%s\" is set but no key"
-					" file is.", certfile);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			return 0;
-		}
-		gnutls_certificate_client_set_retrieve_function(session->cred,
-				&_cb_request_cert);
-	}
-
-	/* put the x509 credentials to the current session */
-	ret = gnutls_credentials_set(session->tls, GNUTLS_CRD_CERTIFICATE,
-				   session->cred);
-	if (ret < 0) {
-		SET_ERROR(err, GNUTLS_ERROR, ret);
-		return 0;
-	}
-	session->need_set_cred = 0;
 	return 1;
 }
+
+
+/**
+ * \ingroup nuclientAPI
+ * Initialize TLS:
+ *    - Set trust file of credentials (if needed)
+ *
+ * \param session Pointer to client session
+ * \param cafile Complete path to a certificate authority file stored in PEM format (can be NULL)
+ * \param err Pointer to a nuclient_error_t: which contains the error
+ * \return Returns 0 on error (error description in err), 1 otherwise
+ */
+int nu_client_load_ca(nuauth_session_t * session,
+			const char *cafile,
+			nuclient_error_t * err)
+{
+	char castring[256];
+	char *home = nu_get_home_dir();
+	int exit_on_error = 0;
+	int ret;
+
+	if (cafile != NULL)
+		exit_on_error = 1;
+
+	if (cafile == NULL && home != NULL) {
+		ret = secure_snprintf(castring, sizeof(castring),
+				     "%s/.nufw/cacert.pem", home);
+		if (ret)
+			cafile = castring;
+	}
+
+	if (cafile != NULL) {
+		ret = nussl_ssl_trust_cert_file(session->nussl, cafile);
+		if (ret != NUSSL_OK) {
+			if (exit_on_error) {
+				if (home)
+					free(home);
+				SET_ERROR(err, NUSSL_ERR, ret);
+				return 0;
+			}
+			else {
+				if (!session->suppress_ca_warning) {
+					log_printf(DEBUG_LEVEL_WARNING, "\nWARNING: you have not provided any certificate authority.\n"
+							"nutcpc will *NOT* verify server certificate trust.\n"
+							"Use the -A <cafile> option to set up CA.\n"
+					       );
+				}
+				session->suppress_fqdn_verif = 1;
+				nussl_set_session_flag(session->nussl, NUSSL_SESSFLAG_IGNORE_ID_MISMATCH, 1);
+			}
+		}
+	} else {
+		log_printf(DEBUG_LEVEL_WARNING, "Could not load any CA !");
+		return 0;
+	}
+	return 1;
+}
+
+int nu_client_load_crl(nuauth_session_t *session, const char *crlfile,
+	const char *cafile, nuclient_error_t * err)
+{
+	int ret;
+	if (crlfile && *crlfile) {
+		ret = nussl_ssl_set_crl_file(session->nussl, crlfile, cafile);
+		if (ret != NUSSL_OK) {
+			log_printf(DEBUG_LEVEL_WARNING,"TLS error with CRL: %s",
+				nussl_get_error(session->nussl));
+			return 0;
+		}
+		log_printf(DEBUG_LEVEL_DEBUG, "Using crl: %s", crlfile);
+	}
+	return 1;
+}
+
+/**
+ * \ingroup nuclientAPI
+ * Returns a formatted string containing information about the TLS cipher
+ * used for the connection to the server.
+ *
+ * \param session Pointer to client session
+ * \return
+ */
+char* nu_client_get_cipher(nuauth_session_t * session)
+{
+	char buf[256];
+	int ret;
+
+	ret = nussl_session_get_cipher(session->nussl, buf, sizeof(buf));
+	return (ret==0) ? strdup(buf) : NULL;
+}
+
+/**
+ * \ingroup nuclientAPI
+ * Returns a formatted string containing information about the user certificate
+ *
+ * \param session Pointer to client session
+ * \return
+ */
+char* nu_client_get_cert_info(nuauth_session_t * session)
+{
+	return nussl_get_cert_info(session->nussl);
+}
+
+
+/**
+ * \ingroup nuclientAPI
+ * Returns a formatted string containing information about the server certificate
+ *
+ * \param session Pointer to client session
+ * \return
+ */
+char* nu_client_get_server_cert_info(nuauth_session_t * session)
+{
+	return nussl_get_server_cert_info(session->nussl);
+}
+
+
 /**
  * \ingroup nuclientAPI
  */
@@ -545,20 +618,21 @@ int nu_client_set_nuauth_cert_dn(nuauth_session_t * session,
 	return 1;
 }
 
-/**
- * \ingroup nuclientAPI
- */
+ /**
+  * \ingroup nuclientAPI
+  */
 int nu_client_set_crlfile(nuauth_session_t * session,
-				char *crlfile,
-				nuclient_error_t *err)
+		const char *crlfile,
+		nuclient_error_t *err)
 {
-	if (crlfile && *crlfile) {
-		session->crl_file = crlfile;
-		return 1;
-	}
-	return 0;
-}
+	if (session->pem_crl)
+		free(session->pem_crl);
 
+	if (crlfile)
+		session->pem_crl = strdup(crlfile);
+
+	return 1;
+}
 
 /**
  * \ingroup nuclientAPI
@@ -572,6 +646,38 @@ int nu_client_set_krb5_service(nuauth_session_t * session,
 	return 1;
 }
 
+/**
+ * \ingroup nuclientAPI
+ */
+int nu_client_set_ca_suppress_warning(nuauth_session_t * session,
+				int suppress_ca_warning)
+{
+	session->suppress_ca_warning = suppress_ca_warning;
+	return 1;
+}
+
+/**
+ * \ingroup nuclientAPI
+ */
+int nu_client_set_fqdn_suppress_verif(nuauth_session_t * session,
+				int suppress_fqdn_verif)
+{
+	session->suppress_fqdn_verif = suppress_fqdn_verif;
+	return 1;
+}
+
+/**
+ * \ingroup nuclientAPI
+ */
+int nu_client_set_cert_suppress_verif(nuauth_session_t * session,
+				int suppress_cert_verif)
+{
+	session->suppress_cert_verif = suppress_cert_verif;
+	if (suppress_cert_verif)
+		session-> suppress_fqdn_verif = 1;
+	return 1;
+}
+
 /**
  * \ingroup nuclientAPI
  * Set IP source of the socket used to connect to nuauth server
@@ -585,34 +691,6 @@ void nu_client_set_source(nuauth_session_t *session, struct sockaddr_storage *ad
 	session->src_addr = *addr;
 }
 
-/**
- * \ingroup nuclientAPI
- */
-int nu_client_reset_tls(nuauth_session_t *session)
-{
-	int ret;
-	session->need_set_cred = 1;
-
-	/* Initialize TLS session */
-	ret = gnutls_init(&session->tls, GNUTLS_CLIENT);
-	if (ret != 0) {
-		return 0;
-	}
-
-	ret = gnutls_set_default_priority(session->tls);
-	if (ret < 0) {
-		return 0;
-	}
-
-	ret =
-	    gnutls_certificate_type_set_priority(session->tls,
-						 cert_type_priority);
-	if (ret < 0) {
-		return 0;
-	}
-	return 1;
-}
-
 /**
  * \brief Init connection to nuauth server
  *
@@ -632,8 +710,6 @@ nuauth_session_t *_nu_client_new(nuclient_error_t * err)
 {
 	conntable_t *new;
 	nuauth_session_t *session;
-	int ret;
-
 
 	/* First reset error */
 	SET_ERROR(err, INTERNAL_ERROR, NO_ERR);
@@ -648,24 +724,19 @@ nuauth_session_t *_nu_client_new(nuclient_error_t * err)
 	/* Set basic fields */
 	session->userid = getuid();
 	session->connected = 0;
-	session->count_msg_cond = -1;
 	session->auth_by_default = 1;
 	session->packet_seq = 0;
-	session->checkthread = NULL_THREAD;
-	session->recvthread = NULL_THREAD;
-	session->tls = NULL;
 	session->ct = NULL;
-	session->tls_password = NULL;
 	session->debug_mode = 0;
 	session->verbose = 1;
 	session->timestamp_last_sent = time(NULL);
-	session->need_set_cred = 1;
-	session->need_ca_verif = 0;
-
-	/* create session mutex */
-	pthread_mutex_init(&(session->mutex), NULL);
-	pthread_mutex_init(&(session->check_count_mutex), NULL);
-	pthread_cond_init(&(session->check_cond), NULL);
+	session->min_sleep_delay.tv_sec = MIN_DELAY_SEC;
+	session->min_sleep_delay.tv_usec = MIN_DELAY_USEC;
+	session->max_sleep_delay.tv_sec = MAX_DELAY_SEC;
+	session->max_sleep_delay.tv_usec = MAX_DELAY_USEC;
+	session->sleep_delay.tv_sec = MIN_DELAY_SEC;
+	session->sleep_delay.tv_usec = MIN_DELAY_USEC;
+	session->hash = 0;
 
 	if (tcptable_init(&new) == 0) {
 		SET_ERROR(err, INTERNAL_ERROR, MEMORY_ERR);
@@ -674,19 +745,8 @@ nuauth_session_t *_nu_client_new(nuclient_error_t * err)
 	}
 	session->ct = new;
 
-	/* X509 stuff */
-	ret = gnutls_certificate_allocate_credentials(&(session->cred));
-	if (ret != 0) {
-		SET_ERROR(err, GNUTLS_ERROR, ret);
-		nu_exit_clean(session);
-		return NULL;
-	}
+	nu_client_set_client_info(session, "unknown client", "unknown version");
 
-	if (!nu_client_reset_tls(session))
-	{
-		SET_ERROR(err, GNUTLS_ERROR, ret);
-		nu_exit_clean(session);
-	}
 	return session;
 }
 
@@ -754,6 +814,7 @@ nuauth_session_t *nu_client_new(const char *username,
 
 	session->username = secure_str_copy(username);
 	session->password = secure_str_copy(password);
+	session->nu_capabilities[0] = 0;
 	if (session->username == NULL || session->password == NULL) {
 		SET_ERROR(err, INTERNAL_ERROR, MEMORY_ERR);
 		return NULL;
@@ -769,30 +830,88 @@ nuauth_session_t *nu_client_new(const char *username,
  */
 void nu_client_reset(nuauth_session_t * session)
 {
-	/* close TLS conneciton */
 	ask_session_end(session);
 
-	/* delete old TLS session and create a new TLS session */
-	gnutls_deinit(session->tls);
-	gnutls_init(&session->tls, GNUTLS_CLIENT);
-	gnutls_set_default_priority(session->tls);
-	gnutls_certificate_type_set_priority(session->tls,
-					     cert_type_priority);
-	session->need_set_cred = 1;
-
-	/* close socket */
-	if (session->socket > 0) {
-		shutdown(session->socket, SHUT_WR);
-		close(session->socket);
-	}
-
 	/* reset fields */
 	session->connected = 0;
-	session->count_msg_cond = -1;
 	session->timestamp_last_sent = time(NULL);
-	session->socket = -1;
-	session->checkthread = 0;
-	session->recvthread = 0;
+}
+
+static int finish_init(nuauth_session_t * session, nuclient_error_t * err)
+{
+	int finish = 0;
+	int ret;
+	char buf[1024];
+	int bufsize;
+	struct nu_srv_message * message = (struct nu_srv_message *) buf;
+
+	while (! finish) {
+		bufsize = nussl_read(session->nussl, buf, sizeof(buf));
+		if ((bufsize <= 0) ||
+			((size_t)bufsize < sizeof(struct nu_srv_message))) {
+			/* allo houston */
+			return 0;
+		}
+		switch (message->type) {
+			case SRV_REQUIRED_INFO:
+				switch (message->option) {
+					case OS_VERSION:
+						if (!send_os(session, err)) {
+							return 0;
+						}
+						break;
+					case CLIENT_VERSION:
+						if (!send_client(session, err)) {
+							return 0;
+						}
+						break;
+					case CLIENT_CAPA:
+						if (!send_capa(session, err)) {
+							return 0;
+						}
+						break;
+					default:
+						return 0;
+				}
+				break;
+			case SRV_TYPE:
+				switch (message->option) {
+					case SRV_TYPE_POLL:
+					case SRV_TYPE_PUSH:
+						session->server_mode = message->option;
+						break;
+					case SRV_HASH_TYPE:
+						session->hash = ntohs(message->length);
+						break;
+					default:
+						break;
+				}
+				break;
+			case SRV_EXTENDED_PROTO:
+				ret = process_ext_message(buf + sizeof(struct nu_srv_message),
+						bufsize - sizeof(struct nu_srv_message),
+						&nu_postauth_extproto_l,
+						session);
+				if (ret != 0) {
+					return 0;
+				}
+				break;
+			case SRV_INIT:
+				finish = 1;
+				switch (message->option) {
+					case INIT_NOK:
+						SET_ERROR(err, INTERNAL_ERROR, NUFW_INITNEGO_ERR);
+						return 0;
+					case INIT_OK:
+						session->connected = 1;
+						break;
+				}
+				break;
+		}
+	}
+
+	return 1;
+
 }
 
 /**
@@ -814,60 +933,54 @@ int nu_client_connect(nuauth_session_t * session,
 		      nuclient_error_t * err)
 {
 	int ret;
-	if (session->need_set_cred) {
-		/* put the x509 credentials to the current session */
-		ret = gnutls_credentials_set(session->tls,
-					     GNUTLS_CRD_CERTIFICATE,
-					     session->cred);
-		if (ret < 0) {
-			SET_ERROR(err, GNUTLS_ERROR, ret);
+	unsigned int port = atoi(service);
+
+	session->nussl = nussl_session_create(NUSSL_SSL_CTX_CLIENT);
+
+	if (session->suppress_cert_verif)
+		nussl_ssl_disable_certificate_check(session->nussl,1);
+
+	if (session->suppress_fqdn_verif)
+		nussl_set_session_flag(session->nussl, NUSSL_SESSFLAG_IGNORE_ID_MISMATCH, 1);
+
+	nussl_set_hostinfo(session->nussl, hostname, port);
+	if (session->pkcs12_file) {
+		if (!nu_client_load_pkcs12(session, session->pkcs12_file, session->pkcs12_password, err))
+			return 0;
+	} else {
+		if (!nu_client_load_key(session, session->pem_key, session->pem_cert, err))
 			return 0;
-		}
-		session->need_set_cred = 0;
 	}
 
-	if (session->crl_file) {
-		struct stat stats;
-		if (stat(session->crl_file, &stats) < 0) {
-			nu_error_log(err, "Can't access CRL file \"%s\"",
-					session->crl_file);
-			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-			return 0;
-		}
-		session->crl_file_mtime = stats.st_mtime;
-
-		if (check_crl_validity(session->crl_file, session->ca_file, err) == -1) {
-			return 0;
-		}
-		ret = gnutls_certificate_set_x509_crl_file(session->cred,
-							session->crl_file,
-							GNUTLS_X509_FMT_PEM);
-		if (ret < 0) {
-			SET_ERROR(err, GNUTLS_ERROR, ret);
-			nu_error_log(err, "CRL file (name %s) problem: %s",
-				session->crl_file,
-				gnutls_strerror(ret));
-			return 0;
-		}
-	}
-
-	/* set field about host */
-	if (!init_socket(session, hostname, service, err)) {
+	if (!nu_client_load_ca(session, session->pem_ca, err))
 		return 0;
+
+	if (session->pem_crl) {
+		if (!nu_client_load_crl(session, session->pem_crl, session->pem_ca, err))
+			return 0;
 	}
 
-	if (!tls_handshake(session, hostname, err)) {
+	ret = nussl_open_connection(session->nussl);
+	if (ret != NUSSL_OK) {
+		log_printf(DEBUG_LEVEL_CRITICAL, "%s", nussl_get_error(session->nussl));
+		nussl_session_destroy(session->nussl);
+		session->nussl = NULL;
+		SET_ERROR(err, NUSSL_ERR, ret);
 		return 0;
 	}
 
 	if (!init_sasl(session, hostname, err)) {
+		plugin_emit_event(NUCLIENT_EVENT_LOGIN_FAILED, session, session->username);
 		return 0;
 	}
 
-	if (!send_os(session, err)) {
+	if (!finish_init(session, err)) {
+		plugin_emit_event(NUCLIENT_EVENT_LOGIN_FAILED, session, session->username);
 		return 0;
 	}
-	session->connected = 1;
+
+	plugin_emit_event(NUCLIENT_EVENT_LOGIN_OK, session, session->username);
+
 	return 1;
 }
 
@@ -896,6 +1009,34 @@ void nu_client_set_verbose(nuauth_session_t * session, unsigned char enabled)
 	session->verbose = enabled;
 }
 
+/**
+ * \ingroup nuclientAPI
+ * Set minimum delay
+ *
+ * \param session Pointer to client session
+ * \param delay a timeval which will be equal to the minimum delay
+ * between two checks (in ms)
+ */
+void nu_client_set_min_delay(nuauth_session_t * session, unsigned int delay)
+{
+	session->min_sleep_delay.tv_sec = delay / 1000;
+	session->min_sleep_delay.tv_usec = (delay * 1000) % 1000000;
+}
+
+/**
+ * \ingroup nuclientAPI
+ * Set maximum delay
+ *
+ * \param session Pointer to client session
+ * \param delay a timeval which will be equal to the maximum delay
+ * between two checks (in ms)
+ */
+void nu_client_set_max_delay(nuauth_session_t * session, unsigned int delay)
+{
+	session->max_sleep_delay.tv_sec = delay / 1000;
+	session->max_sleep_delay.tv_usec = (delay * 1000) % 1000000;
+}
+
 /**
  * \ingroup nuclientAPI
  * \brief Allocate a structure to store client error
@@ -907,7 +1048,6 @@ int nu_client_error_init(nuclient_error_t ** err)
 	*err = malloc(sizeof(nuclient_error_t));
 	if (*err == NULL)
 		return -1;
-	(*err)->strerror[0] = 0;
 	return 0;
 }
 
@@ -925,17 +1065,17 @@ void nu_client_error_destroy(nuclient_error_t * err)
  * \ingroup nuclientAPI
  * \brief Convert an error to an human readable string
  */
-const char *nu_client_strerror(nuclient_error_t * err)
+const char *nu_client_strerror(nuauth_session_t * session, nuclient_error_t * err)
 {
-	if (err == NULL)
+	if (err == NULL) {
 		return "Error structure was not initialised";
-	if (err->strerror[0]) {
-		return err->strerror;
 	}
+
 	switch (err->family) {
-	case GNUTLS_ERROR:
-		return gnutls_strerror(err->error);
-		break;
+	case NUSSL_ERR:
+		if (session == NULL || session->nussl == NULL)
+			return "NuSSL initialization error.";
+		return nussl_get_error(session->nussl);
 	case SASL_ERROR:
 		return sasl_errstring(err->error, NULL, NULL);
 		break;
@@ -965,8 +1105,14 @@ const char *nu_client_strerror(nuclient_error_t * err)
 			return "Bad credentials";
 		case BINDING_ERR:
 			return "Binding (source address) error";
-		case TLS_HANDSHAKE_ERR:
-			return "TLS handshake failed";
+		case NUSSL_INIT_ERR:
+			return "NuSSL initialisation failed.";
+		case NUFW_INITNEGO_ERR:
+			return "NuFW refused connection during init.";
+		case NUFW_CRUISE_ERR:
+			return "NuFW error during cruise protocol.";
+		case PROTO_ERR:
+			return "Protocol error (too old authentication server ?).";
 		default:
 			return "Unknown internal error code";
 		}
@@ -1002,5 +1148,4 @@ int nu_check_version(const char *version)
 		return 0;
 }
 
-
 /** @} */
