@@ -4,12 +4,12 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr
  **
- ** $Id: tcptable.c 5225 2008-11-19 14:29:23Z regit $
+ ** $Id$
  **
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,19 +20,9 @@
  ** along with this program; if not, write to the Free Software
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-
-#include <config.h>
-#ifdef FREEBSD
-#include "ipv6.h"
 #include "libnuclient.h"
-#else
-#include "libnuclient.h"
-#include "ipv6.h"
-#endif
-
-#include "client.h"
 #include "proto.h"
-#include <pthread.h>
+#include "tcptable.h"
 #include <inttypes.h>
 #define USE_JHASH3
 #include <jhash.h>
@@ -52,6 +42,8 @@
 
 #endif
 
+#include <nubase.h>
+
 /**
  * \addtogroup libnuclient
  * @{
@@ -103,7 +95,7 @@ int parse_tcptable_file(nuauth_session_t * session, conntable_t * ct, char *file
 
 	/* convert session user identifier to string */
 	secure_snprintf(session_uid, sizeof(session_uid), "%5lu",
-			session->userid);
+			(long)session->userid);
 	session_uid_len = strlen(session_uid);
 
 	/* get state field position in header */
@@ -210,12 +202,6 @@ int tcptable_read(nuauth_session_t * session, conntable_t * ct)
 	assert(ct != NULL);
 	assert(TCP_SYN_SENT == 2);
 #endif
-	if (session->server_mode == SRV_TYPE_PUSH) {
-		/* need to set check_cond */
-		pthread_mutex_lock(&(session->check_count_mutex));
-		session->count_msg_cond = 0;
-		pthread_mutex_unlock(&(session->check_count_mutex));
-	}
 
 	if (!parse_tcptable_file
 	    (session, ct, "/proc/net/tcp", &fd_tcp, IPPROTO_TCP, 0))
@@ -266,13 +252,6 @@ int tcptable_read(nuauth_session_t * session, conntable_t * ct)
 		return 0;
 	}
 
-	if (session->server_mode == SRV_TYPE_PUSH) {
-		/* need to set check_cond */
-		pthread_mutex_lock(&(session->check_count_mutex));
-		session->count_msg_cond = 0;
-		pthread_mutex_unlock(&(session->check_count_mutex));
-	}
-
 	/* read connection table */
 	if (sysctlbyname(mibvar, buf, &len, 0, 0) < 0) {
 		printf("sysctl: %s", mibvar);
@@ -349,7 +328,7 @@ int tcptable_init(conntable_t ** ct)
  * Compute connection hash (index in a connection table, see ::conntable_t).
  * Hash is an integer in interval 0..(::CONNTABLE_BUCKETS-1).
  */
-inline int tcptable_hash(conn_t * c)
+int tcptable_hash(conn_t * c)
 {
 	/** \todo Hash the whole ip address! */
 #ifndef FREEBSD
