@@ -1,14 +1,12 @@
 /*
- ** Copyright 2004-2008 - INL
- ** Written by Eric Leblond <eric@inl.fr>
+ ** Copyright 2004-2009 - INL
+ ** Written by Eric Leblond <eric.leblond@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nutcpc.c 5468 2008-12-31 10:34:06Z regit $
- **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,27 +18,44 @@
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Enable GNU extensions: getline() from stdio.h */
 #include "nufw_source.h"
 
-#include "../lib/nuclient.h"
-#include <locale.h>
 #include <config.h>
+#include "../lib/nuclient.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <errno.h>
+#include <locale.h>
 #include <sys/resource.h>	/* setrlimit() */
 #include <langinfo.h>
 #include <stdarg.h>
 #include <signal.h>
+#include <sys/ioctl.h>
+#include <sys/stat.h>	/* mkdir() */
+#include <sys/types.h>	/* mkdir() */
+#include <termios.h>	/* tcgetattr() */
 #include "proto.h"
 #include "security.h"
-#define NUTCPC_VERSION PACKAGE_VERSION " $Revision: 5468 $"
+#include "debug.h"
+#define NUTCPC_VERSION PACKAGE_VERSION " $Revision$"
 
-#define RETURN_NO_LOG return
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#endif
 
 #ifdef FREEBSD
+#include <sys/syslimits.h>	/* PATH_MAX */
 #include <readpassphrase.h>
 #endif
 
 #define MAX_RETRY_TIME 30
 
+#include <nubase.h>
+#include <nussl.h>
+
 struct termios orig;
 nuauth_session_t *session = NULL;
 nuclient_error_t *err = NULL;
@@ -50,6 +65,15 @@ int forced_reconnect = 0;
 int connected;
 static int suppress_ca_warning = 0;
 static int suppress_fqdn_verif = 0;
+static int do_not_load_config  = 0;
+static int do_not_load_plugins = 0;
+static int mono_user = 0;
+
+void panic(const char *fmt, ...)
+#ifdef __GNUC__
+	__attribute__((__format__(printf,1,2)))
+#endif
+;
 
 typedef struct {
 	char port[10];		/*!< Port (service) number / name */
@@ -64,7 +88,8 @@ typedef struct {
 	char *keyfile;
 	char *cafile;
 	char *crlfile;
-	char *cert_password;
+	char *pkcs12file;
+	char *pkcs12password;
 	char *krb5_service;
 } nutcpc_context_t;
 
@@ -107,10 +132,7 @@ char *compute_run_pid()
 }
 
 /**
- * Kill existing instance of nutcpc: read pid file,
- * and then send SIGTERM to the process.
- *
- * Exit the program at the end of this function.
+ * Test if a nutcpc is currently running
  */
 int test_nutcpc(pid_t *pid)
 {
@@ -382,38 +404,76 @@ char *get_user_validation(const char *msg)
 	return answer;
 }
 
+static struct option long_options[] = {
+	{"help", 0, NULL, 'h'},
+	{"user", 1, NULL, 'U'},
+	{"host", 1, NULL, 'H'},
+	{"kill", 0, NULL, 'k'},
+	{"check", 0, NULL, 'c'},
+	{"no-lock", 0, NULL, 'l'},
+	{"version", 0, NULL, 'V'},
+	{"cert", 1, NULL, 'C'},
+	{"ca", 1, NULL, 'A'},
+	{"key", 1, NULL, 'K'},
+	{"pkcs12-file", 1, NULL, 'S'},
+	{"pkcs12-key", 1, NULL, 'W'},
+	{"crl", 1, NULL, 'R'},
+	{"no-warn-ca", 0, NULL, 'Q'},
+	{"no-error-fqdn", 0, NULL, 'N'},
+	{"no-config", 0, NULL, 'F'},
+	{"no-plugins", 0, NULL, 'G'},
+	{"krb-service", 1, NULL, 'Z'},
+	{"port", 1, NULL, 'p'},
+	{"auth-dn", 1, NULL, 'a'},
+	{"interval", 1, NULL, 'I'},
+	{"hide", 0, NULL, 'q'},
+	{"password", 1, NULL, 'P'},
+	{"debug", 0, NULL, 'd'},
+	{"mono", 0, NULL, 'm'},
+
+	{0, 0, 0, 0}
+};
+
+
 /**
  * Print client usage.
  */
 static void usage(void)
 {
-	fprintf(stderr, "usage: nutcpc -U username -H host\n");
+	fprintf(stderr, "usage: nutcpc [-U username] [-H host]\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Options:\n");
-	fprintf(stderr, "  -k: kill active client\n");
-	fprintf(stderr, "  -c: check if there is an active client\n");
-	fprintf(stderr, "  -l: don't create lock file\n");
-	fprintf(stderr, "  -V: display version\n");
+	fprintf(stderr, "  -U (--user         ): username (default: current login)\n");
+	fprintf(stderr, "  -H (--host         ): nuauth server\n");
+	fprintf(stderr, "  -k (--kill         ): kill active client\n");
+	fprintf(stderr, "  -c (--check        ): check if there is an active client\n");
+	fprintf(stderr, "  -l (--no-lock      ): don't create lock file\n");
+	fprintf(stderr, "  -m (--mono         ): mono user system\n");
+	fprintf(stderr, "  -V (--no-version   ): display version\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Certificate options:\n");
-	fprintf(stderr, "  -C CERTFILE: certificate filename\n");
-	fprintf(stderr, "  -A AUTHFILE: authority certificate filename\n");
-	fprintf(stderr, "  -K KEYFILE:  key filename\n");
-	fprintf(stderr, "  -W CERTPASS: certificate password\n");
-	fprintf(stderr, "  -R CRLFILE: crl filename\n");
-	fprintf(stderr, "  -Q: suppress warning if no certificate authority is configured\n");
-	fprintf(stderr, "  -N: suppress error if server FQDN does not match certificate CN.\n");
+	fprintf(stderr, "  -C (--cert         ) CERTFILE: PEM certificate filename\n");
+	fprintf(stderr, "  -A (--ca           ) AUTHFILE: PEM authority certificate filename\n");
+	fprintf(stderr, "  -K (--key          ) KEYFILE:  PEM RSA private key filename\n");
+	fprintf(stderr, "  -S (--pkcs12-file  ) PKCS12FILE: PKCS12 key/certificate filename\n");
+	fprintf(stderr, "  -W (--pkcs12-key   ) PKCS12PASS: PKCS12 password\n");
+	fprintf(stderr, "  -R (--crl          ) CRLFILE: crl filename\n");
+	fprintf(stderr, "  -Q (--no-warn-ca   ): suppress warning if no certificate authority is configured\n");
+	fprintf(stderr, "  -N (--no-error-fqdn): suppress error if server FQDN does not match certificate CN.\n");
+	fprintf(stderr, "  -F (--no-config    ): do not load config file (implies -G).\n");
+	fprintf(stderr, "  -G (--no-plugins   ): do not loadplugins.\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "SASL options:\n");
-	fprintf(stderr, "  -Z SERVICE: Kerberos service name (nuauth)\n");
+	fprintf(stderr, "  -Z (--krb-service  ) SERVICE: Kerberos service name (nuauth)\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Other options:\n");
-	fprintf(stderr, "  -p PORT: nuauth port number\n");
-	fprintf(stderr, "  -a AUTH_DN: authentication domain name\n");
-	fprintf(stderr, "  -I INTERVAL: check interval in milliseconds\n");
-	fprintf(stderr, "  -q: do not display running nutcpc options on \"ps\"\n");
-	fprintf(stderr, "  -P PASSWORD: specify password (only for debug purpose)\n");
-	fprintf(stderr, "  -d: debug mode (don't go to foreground, daemon)\n");
+	fprintf(stderr, "  -p (--port         ) PORT: nuauth port number\n");
+	fprintf(stderr, "  -a (--auth-dn      ) AUTH_DN: authentication domain name\n");
+	fprintf(stderr, "  -I (--interval     ) INTERVAL: check interval in milliseconds\n");
+	fprintf(stderr, "  -P (--password     ):PASSWORD: specify password (only for debug purpose)\n");
+	fprintf(stderr, "  -q (--hide         ): do not display running nutcpc options on \"ps\"\n");
+	fprintf(stderr, "  -v (--verbose      ): increase debug level (+1 for each 'v') (max useful number : 10)\n");
+	fprintf(stderr, "  -d (--debug        ): debug mode (don't go to foreground, daemon)\n");
 	fprintf(stderr, "\n");
 	exit(EXIT_FAILURE);
 }
@@ -513,6 +573,26 @@ void wipe(void *data, size_t datalen)
 	memset(data, 0, datalen);
 }
 
+/**
+ * Display informations about the user certificate
+ *
+ */
+void display_cert(nuauth_session_t* session)
+{
+	char* info;
+
+	info = nu_client_get_cipher(session);
+	printf("Server cipher:\n%s\n", info ? info : "None");
+	free(info);
+	info = nu_client_get_cert_info(session);
+	printf("User certificate:\n%s\n", info ? info : "None");
+	free(info);
+	info = nu_client_get_server_cert_info(session);
+	printf("Server certificate:\n%s\n", info ? info : "None");
+	free(info);
+}
+
+
 /**
  * Try to connect to nuauth.
  *
@@ -536,6 +616,8 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 		nu_client_set_password(session, context->password);
 	}
 
+	nu_client_set_client_info(session, "nutcpc", NUTCPC_VERSION);
+
 	nu_client_set_debug(session, context->debug_mode);
 
 	/* Set hostname from libnuclient if it wasn't specified by the user */
@@ -559,43 +641,56 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 		}
 	}
 
-	printf("Connecting to NuFW gateway (%s)\n", context->srv_addr);
-
-	if (!nu_client_setup_tls(session, context->keyfile, context->certfile,
-	     context->cafile, context->cert_password, err)) {
-		nu_client_delete(session);
-		fprintf(stderr, "Unable to setup TLS params\n");
-		return NULL;
+	if (context->pkcs12file) {
+		if (!nu_client_set_pkcs12(session, context->pkcs12file, context->pkcs12password, err)) {
+			goto init_failed;
+		}
 	}
-
-	if (context->crlfile) {
-		if (!nu_client_set_crlfile(session, context->crlfile, err)) {
-			nu_client_delete(session);
-			fprintf(stderr, "Error when setting crlfile: %s\n",
-				nu_client_strerror(err));
-			return NULL;
+	else {
+		if (!nu_client_set_key(session, context->keyfile, context->certfile, err)) {
+			goto init_failed;
 		}
 	}
 
-	session->need_fqdn_verif = !suppress_fqdn_verif;
+	if (!context->cafile) {
+		if (!suppress_ca_warning) {
+			char *reply;
 
-	if (session->need_ca_verif == 0) {
-		if (suppress_ca_warning == 0)
-			fprintf(stderr,"\nWARNING: you have not provided any certificate authority.\n"
-				"nutcpc will *NOT* verify server certificate trust.\n"
-				"Use the -A <cafile> option to set up CA.\n"
-				"As certificate will not be trusted, disabling FQDN check.\n\n"
-				);
-		session->need_fqdn_verif = 0;
+			reply = get_user_validation(
+					"*******   WARNING   ******\n"
+					"You are trying to connect to nuauth without configuring a certificate authority (CA).\n"
+					"You are vulnerable to attacks like man-in-the-middle.\n"
+					"Do you really want to do that ? Type \"yes\" to continue: ");
+
+			if (reply==NULL || strcasecmp(reply,"YES")!=0) {
+				fprintf(stderr,"Aborted");
+				goto init_failed;
+			}
+			free(reply);
+		}
+		nu_client_set_cert_suppress_verif(session, 1);
 	}
+	if (!nu_client_set_ca(session, context->cafile, err)) {
+		goto init_failed;
+	}
+
+	nu_client_set_ca_suppress_warning(session,suppress_ca_warning);
+	if (suppress_fqdn_verif)
+		nu_client_set_fqdn_suppress_verif(session, 1);
 
 	if (context->nuauthdn) {
 		if (!nu_client_set_nuauth_cert_dn(session,
 						  context->nuauthdn,
 						  err)) {
-			nu_client_delete(session);
-			fprintf(stderr, "Unable to setup nuauth cert DN\n");
-			return NULL;
+			goto init_failed;
+		}
+	}
+
+	if (!context->crlfile)
+		context->crlfile = (char *)nu_client_default_tls_crl();
+	if (context->crlfile) {
+		if (!nu_client_set_crlfile(session, context->crlfile, err)) {
+			goto init_failed;
 		}
 	}
 
@@ -607,13 +702,15 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 		}
 	}
 
-	if (!nu_client_connect
-	    (session, context->srv_addr, context->port, err)) {
-		fprintf(stderr, "Unable to setup connect\n");
-		nu_client_delete(session);
-		return NULL;
+	if (!nu_client_connect(session, context->srv_addr, context->port, err)) {
+		goto init_failed;
 	}
 	return session;
+init_failed:
+
+	printf("Initialization error: %s\n", nu_client_strerror(session, err));
+	nu_client_delete(session);
+	return NULL;
 }
 
 /**
@@ -624,9 +721,6 @@ void main_loop(nutcpc_context_t * context)
 	int connected = 1;
 	int ret;
 	for (;;) {
-		if (forced_reconnect == 0) {
-			usleep(context->interval * 1000);
-		}
 		if (!connected) {
 			if (forced_reconnect == 0) {
 				usleep((unsigned long) context->tempo * 1000000);
@@ -646,7 +740,7 @@ void main_loop(nutcpc_context_t * context)
 				context->tempo = 1;	/* second */
 			} else {
 				printf("Reconnection error: %s\n",
-				       nu_client_strerror(err));
+						nu_client_strerror(session, err));
 				nu_client_reset(session);
 			}
 		} else {
@@ -656,7 +750,7 @@ void main_loop(nutcpc_context_t * context)
 				/* on error: reset the session */
 				nu_client_reset(session);
 				connected = 0;
-				printf("%s\n", nu_client_strerror(err));
+				printf("%s\n", nu_client_strerror(session, err));
 			}
 		}
 	}
@@ -709,7 +803,7 @@ void parse_cmdline_options(int argc, char **argv,
 
 	/* Parse all command line arguments */
 	opterr = 0;
-	while ((ch = getopt(argc, argv, "klcdqNQVH:I:U:p:P:a:K:C:A:R:W:Z:")) != -1) {
+	while ((ch = getopt_long(argc, argv, "kcldqNQFGVvmu:H:I:U:p:P:a:K:C:A:R:W:S:Z:", long_options, NULL)) != -1) {
 		switch (ch) {
 		case 'H':
 			SECURE_STRNCPY(context->srv_addr, optarg,
@@ -723,6 +817,9 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'd':
 			context->debug_mode = 1;
 			break;
+		case 'v':
+			debug_level++;
+			break;
 		case 'I':
 			context->interval = atoi(optarg);
 			if (context->interval == 0) {
@@ -766,6 +863,16 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'Q':
 			suppress_ca_warning = 1;
 			break;
+		case 'F':
+			do_not_load_config = 1;
+			do_not_load_plugins = 1;
+			break;
+		case 'G':
+			do_not_load_plugins = 1;
+			break;
+		case 'm':
+			mono_user = 1;
+			break;
 		case 'a':
 			SECURE_STRNCPY(context->nuauthdn, optarg,
 				       sizeof(context->nuauthdn));
@@ -782,8 +889,11 @@ void parse_cmdline_options(int argc, char **argv,
 		case 'R':
 			context->crlfile = copy_filename(optarg);
 			break;
+		case 'S':
+			context->pkcs12file = copy_filename(optarg);
+			break;
 		case 'W':
-			context->cert_password = strdup(optarg);
+			context->pkcs12password = strdup(optarg);
 			break;
 		case 'Z':
 			context->krb5_service = strdup(optarg);
@@ -795,7 +905,13 @@ void parse_cmdline_options(int argc, char **argv,
 	if (context->password[0] != 0 && !context->debug_mode) {
 		fprintf(stderr,
 			"Don't use -P option outside debugging, it's not safe!\n");
-		exit(1);
+		exit(EXIT_FAILURE);
+	}
+
+	if ((context->keyfile || context->certfile) && (context->pkcs12file || context->pkcs12password))
+	{
+		fprintf(stderr, "Don't mix PKCS12 options with X509/RSA options.\n");
+		exit(EXIT_FAILURE);
 	}
 
 	/* fill argument with nul byte */
@@ -807,18 +923,6 @@ void parse_cmdline_options(int argc, char **argv,
 	}
 }
 
-/**
- * Stupid wrapper
- */
-const char *wrap_error_message(const char *str)
-{
-	if (strncmp(str,"A TLS packet with unexpected length was received.",49) == 0) {
-		RETURN_NO_LOG("Access denied");
-	}
-	return(str);
-}
-
-
 /**
  * Initialize nuclient library
  */
@@ -827,7 +931,7 @@ void init_library(nutcpc_context_t * context, char *username)
 	struct rlimit core_limit;
 
 	/* Avoid creation of core file which may contains username and password */
-	if (getrlimit(RLIMIT_CORE, &core_limit) == 0) {
+	if (!context->debug_mode && getrlimit(RLIMIT_CORE, &core_limit) == 0) {
 		core_limit.rlim_cur = 0;
 		setrlimit(RLIMIT_CORE, &core_limit);
 	}
@@ -841,9 +945,17 @@ void init_library(nutcpc_context_t * context, char *username)
 	/* global libnuclient init */
 	if (!nu_client_global_init(err)) {
 		printf("Unable to initiate nuclient library!\n");
-		printf("Problem: %s\n", nu_client_strerror(err));
+		printf("Problem: %s\n", nu_client_strerror(session, err));
 		exit(EXIT_FAILURE);
 	}
+
+	if (do_not_load_config != 1) {
+		nu_client_init_config();
+
+		if (do_not_load_plugins != 1)
+			nu_client_init_plugins();
+	}
+
 	/* options specificied on command line are taken prior
 	 * to options from configuration file
 	 */
@@ -854,35 +966,24 @@ void init_library(nutcpc_context_t * context, char *username)
 		context->certfile = (char *)nu_client_default_tls_cert();
 	if (!context->keyfile)
 		context->keyfile = (char *)nu_client_default_tls_key();
-	if (!context->crlfile)
-		context->crlfile = (char *)nu_client_default_tls_crl();
 
-	if (!suppress_ca_warning && !context->cafile) {
-		char *reply;
-
-		reply = get_user_validation(
-				"*******   WARNING   ******\n"
-				"You are trying to connect to nuauth without configuring a certificate authority (CA).\n"
-				"You are vulnerable to attacks like man-in-the-middle.\n"
-				"Do you really want to do that ? Type \"yes\" to continue: ");
-
-		if (reply==NULL || strcasecmp(reply,"YES")!=0) {
-			fprintf(stderr,"Aborted");
-			exit(EXIT_FAILURE);
-		}
-		free(reply);
-
-		/* continue, libnuclient does not require a CA */
+	if (mono_user) {
+		nu_client_set_capability(NU_HELLO_CAPABILITIES);
 	}
 
 	/* Init. library */
+	printf("Connecting to NuFW gateway (%s)\n", context->srv_addr);
 	session = do_connect(context, username);
 
+	if (session) {
+		display_cert(session);
+	}
+
 	/* Library failure? */
 	if (session == NULL) {
 		printf("Unable to initiate connection to NuFW gateway\n");
 		if (err->error != 0)
-			printf("Problem: %s\n", wrap_error_message(nu_client_strerror(err)));
+			printf("Problem: %s\n", nu_client_strerror(session, err));
 		printf("Authentication failed (check parameters)\n");
 		exit(EXIT_FAILURE);
 	}
@@ -892,12 +993,19 @@ int main(int argc, char **argv)
 {
 	char *runpid = compute_run_pid();
 	char *username = NULL;
+	char *default_username = NULL;
 	nutcpc_context_t context;
 	memset(&context, 0, sizeof(context));
 
+	default_username = nu_get_user_name();
+
 	/* needed by iconv */
 	setlocale(LC_ALL, "");
 
+	log_engine = LOG_TO_STD;
+	debug_areas = DEFAULT_DEBUG_AREAS;
+	debug_level = DEBUG_LEVEL_SERIOUS_MESSAGE;
+
 	if (!nu_check_version(NUCLIENT_VERSION)) {
 		fprintf(stderr,
 			"Wrong version of libnuclient (%s instead of %s)\n",
@@ -927,12 +1035,10 @@ int main(int argc, char **argv)
 						    (pid_t) atoi(line);
 						fclose(fd);
 						if (kill(pid, 0)) {
-							printf
-							    ("No running process, starting anyway (deleting lockfile)\n");
+							printf("No running process, starting anyway (deleting lockfile)\n");
 							unlink(runpid);
 						} else {
-							printf
-							    ("Kill existing process with \"-k\" or ignore it with \"-l\" option\n");
+							printf("Kill existing process with \"-k\" or ignore it with \"-l\" option\n");
 							exit(EXIT_FAILURE);
 						}
 					}
@@ -943,9 +1049,8 @@ int main(int argc, char **argv)
 
 	install_signals();
 
-	if (!username) {
-		username = nu_get_user_name();
-	}
+	if (!username)
+		username = default_username;
 
 	init_library(&context, username);
 
