@@ -1,14 +1,14 @@
 /*
- ** Copyright(C) 2006-2007 INL
+ ** Copyright(C) 2006-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Victor Stinner <haypo@inl.fr>
  **     INL : http://www.inl.fr/
  **
- ** $Id: auth_hash.c 5201 2008-11-18 16:55:36Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -110,6 +110,30 @@ gboolean tracking_equal(const tracking_t *trck1, const tracking_t *trck2)
 	}
 }
 
+
+
+nu_error_t merge_nufw_session(connection_t * old, connection_t * new)
+{
+	int usage;
+	if (old->tls == new->tls) {
+		return NU_EXIT_OK;
+	}
+
+	/* initiate merge */
+	usage = g_slist_length(old->packet_id);
+	g_atomic_int_exchange_and_add(&(new->tls->usage), usage);
+	if (usage > 1) {
+		g_atomic_int_exchange_and_add(&(old->tls->usage), -usage + 1);
+	}
+	release_nufw_session(old->tls);
+
+	/* swap tls entry */
+	old->tls = new->tls;
+
+	return NU_EXIT_OK;
+}
+
+
 /**
  * Send the a #WARN_MESSAGE to nuauthdatas->tls_push_queue (see ::push_worker()).
  */
@@ -118,6 +142,7 @@ void search_and_push(connection_t * new)
 	/* push data to sender */
 	struct internal_message *message =
 	    g_new0(struct internal_message, 1);
+	auth_pckt_t * pcktdata = g_new0(auth_pckt_t, 1);
 	if (!message) {
 		log_message(CRITICAL, DEBUG_AREA_USER,
 			    "search&push: Couldn't g_new0(). No more memory?");
@@ -128,7 +153,9 @@ void search_and_push(connection_t * new)
 
 	/* duplicate tracking */
 	message->type = WARN_MESSAGE;
-	message->datas = g_memdup(&(new->tracking), sizeof(tracking_t));
+	memcpy(&(pcktdata->header), &(new->tracking), sizeof(tracking_t));
+	duplicate_iface_nfo(&(pcktdata->iface_nfo), &(new->iface_nfo));
+	message->datas = pcktdata;
 	if (message->datas) {
 		g_async_queue_push(nuauthdatas->tls_push_queue, message);
 	} else {
@@ -148,6 +175,10 @@ void search_and_fill_complete_of_authreq(connection_t * new,
 				  "Complete authreq: Adding a packet_id to a connection (id=%u)",
 				  GPOINTER_TO_UINT((new->packet_id)->data)
 				  );
+
+		/* merge TLS */
+		merge_nufw_session(packet, new);
+
 		packet->packet_id =
 		    g_slist_prepend(packet->packet_id,
 				    GUINT_TO_POINTER((new->packet_id)->
@@ -168,12 +199,13 @@ void search_and_fill_complete_of_authreq(connection_t * new,
 		packet->username = new->username;
 		/* application */
 		packet->app_name = new->app_name;
-		packet->app_md5 = new->app_md5;
+		packet->app_sig = new->app_sig;
 		/* system */
 		packet->os_sysname = new->os_sysname;
 		packet->os_release = new->os_release;
 		packet->os_version = new->os_version;
-		packet->client_version = new->client_version;
+		packet->proto_version = new->proto_version;
+		packet->auth_quality = new->auth_quality;
 		/* user cache system */
 		packet->cacheduserdatas = new->cacheduserdatas;
 
@@ -211,12 +243,13 @@ void search_and_fill_complete_of_userpckt(connection_t * new,
 		new->state = AUTH_STATE_COMPLETING;
 		/* application */
 		new->app_name = packet->app_name;
-		new->app_md5 = packet->app_md5;
+		new->app_sig = packet->app_sig;
 		/* system */
 		new->os_sysname = packet->os_sysname;
 		new->os_release = packet->os_release;
 		new->os_version = packet->os_version;
-		new->client_version = packet->client_version;
+		new->proto_version = packet->proto_version;
+		new->auth_quality = packet->auth_quality;
 		/* copy iface info */
 		duplicate_iface_nfo(&(packet->iface_nfo), &(new->iface_nfo));
 
@@ -228,6 +261,9 @@ void search_and_fill_complete_of_userpckt(connection_t * new,
 		packet->tls = new->tls;
 		packet->nufw_version = new->nufw_version;
 		new->tls = NULL;
+		/* copy payload to user pckt */
+		packet->payload_len = new->payload_len;
+		memcpy(packet->payload, new->payload, new->payload_len);
 
 		thread_pool_push(nuauthdatas->acl_checkers, new, NULL);
 		return;		/* don't free connection */
@@ -311,7 +347,7 @@ void search_and_fill_ready(connection_t * new,
 		packet->packet_id =
 		    g_slist_prepend(packet->packet_id,
 				    GUINT_TO_POINTER((new->packet_id)->data));
-		new->state = AUTH_STATE_DONE;	
+		new->state = AUTH_STATE_DONE;
 		break;
 
 	case AUTH_STATE_USERPCKT:
@@ -363,7 +399,7 @@ void search_and_fill_update(connection_t * new, connection_t * packet)
 }
 
 /**
- * Thread created in ::init_nuauthdatas().
+ * Thread created in ::init_nuauthdata().
  * Try to insert a connection in Struct.
  * Fetch datas in connections queue.
  *
