@@ -2,11 +2,11 @@
  ** Copyright(C) 2004-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: authsrv.c 5470 2008-12-31 11:13:39Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,26 +24,39 @@
  * @{
  */
 
+/* } */
+
 /*! \file nuauth/authsrv.c
 	\brief Main file
 
-  It takes care of init stuffs and runs sheduled tasks at a given interval.
-  */
+	It takes care of init stuffs and runs sheduled tasks at a given interval.
+*/
 
 
 #include <auth_srv.h>
-#include <gcrypt.h>
 #include <sasl/saslutil.h>
-#include "gcrypt_init.h"
-#include "tls.h"
 #include "sasl.h"
 #include "security.h"
 #include <sys/resource.h>	/* setrlimit() */
+#include <fcntl.h>		/* O_RDWR */
+
+#include <nussl.h>
+#include <nubase.h>
+
+#include "nuauthconf.h"
+
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#endif
 
 typedef struct {
 	int daemonize;
-	char *nuauth_client_listen_addr;
-	char *nuauth_nufw_listen_addr;
+	int debug_level;
+	char *configfile;
+	char *userpckt_port;
+	char *authreq_port;
+	char *client_srv;
+	char *nufw_srv;
 } command_line_params_t;
 
 int nuauth_running = 1;
@@ -216,10 +229,12 @@ void stop_threads(gboolean wait)
 {
 	log_message(INFO, DEBUG_AREA_MAIN, "Asking threads to stop.");
 
+#ifdef BUILD_NUAUTH_COMMAND
 	/* stop command server */
 	if (nuauthconf->use_command_server) {
 		thread_stop(&nuauthdatas->command_thread);
 	}
+#endif
 
 	/* ask theads to stop */
 	if (nuauthconf->push && nuauthconf->hello_authentication) {
@@ -262,9 +277,11 @@ void stop_threads(gboolean wait)
 		thread_wait_end(&nuauthdatas->search_and_fill_worker);
 	}
 
+#ifdef BUILD_NUAUTH_COMMAND
 	if (nuauthconf->use_command_server) {
 		thread_wait_end(&nuauthdatas->command_thread);
 	}
+#endif
 	if (nuauthconf->push && nuauthconf->hello_authentication && wait) {
 		thread_wait_end(&nuauthdatas->localid_auth_thread);
 	}
@@ -330,8 +347,11 @@ void nuauth_deinit(gboolean soft)
 	log_message(INFO, DEBUG_AREA_MAIN, "Unloading modules");
 	unload_modules();
 
-	log_message(INFO, DEBUG_AREA_MAIN, "Ending TLS and audit");
+#if 0
 	end_tls();
+#endif
+
+	log_message(INFO, DEBUG_AREA_MAIN, "Ending audit");
 	end_audit();
 
 	log_message(INFO, DEBUG_AREA_MAIN, "Freeing memory");
@@ -397,11 +417,10 @@ void nuauth_cleanup(int recv_signal)
 	else if (recv_signal == SIGTERM)
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 			    "[+] Stopping NuAuth server (SIGTERM)");
-	g_message("[+] Stopping NuAuth server");
 
 	nuauth_deinit(TRUE);
 
-	g_message("[+] NuAuth exiting");
+	log_message(FATAL, DEBUG_AREA_MAIN, "[+] NuAuth exiting");
 	exit(EXIT_SUCCESS);
 }
 
@@ -412,8 +431,7 @@ void nuauth_cleanup(int recv_signal)
  *      and then exit
  *    - Set current directory to "/"
  *    - Call setsid()
- *    - Install log handler: call set_glib_loghandlers(),
- *      close stdin, stdout and stderr
+ *    - close stdin, stdout and stderr
  */
 void daemonize()
 {
@@ -428,8 +446,8 @@ void daemonize()
 		if (pf != NULL &&
 		    fscanf(pf, "%d", &pidv) == 1 && kill(pidv, 0) == 0) {
 			fclose(pf);
-			printf
-			    ("pid file exists. Is nuauth already running? Aborting!\n");
+			log_message(FATAL, DEBUG_AREA_MAIN,
+			    "pid file exists. Is nuauth already running? Aborting!\n");
 			exit(EXIT_FAILURE);
 		}
 
@@ -439,8 +457,8 @@ void daemonize()
 
 	pidf = fork();
 	if (pidf < 0) {
-		g_warning("Unable to fork");
-		exit(EXIT_FAILURE);
+		log_message(FATAL, DEBUG_AREA_MAIN, "Unable to fork");
+		exit(EXIT_FAILURE);	/* this should be useless !! */
 	} else {
 		if (pidf > 0) {
 			/* child process */
@@ -459,14 +477,34 @@ void daemonize()
 
 	setsid();
 
-	set_glib_loghandlers();
-
-	/* Close stdin, stdout, stderr. */
-	(void) close(STDIN_FILENO);
-	(void) close(STDOUT_FILENO);
-	(void) close(STDERR_FILENO);
+	/* warning: do not close fd (0 1 2), or this will create problems when trying
+	 * to create child process with a pipe (dup2 fails with error EBADF)
+	 */
+	{
+		int fd = open("/dev/null",O_RDWR);
+		dup2(fd, STDIN_FILENO);
+		dup2(fd, STDOUT_FILENO);
+		dup2(fd, STDERR_FILENO);
+		close(fd);
+	}
 }
 
+static struct option long_options[] = {
+	{"help", 0, NULL, 'h'},
+	{"config", 1, NULL, 'c'},
+	{"daemon", 0, NULL, 'D'},
+	{"version", 0, NULL, 'V'},
+	{"verbose", 0, NULL, 'v'},
+	{"nufw-port", 1, NULL, 'p'},
+	{"client-port", 1, NULL, 'l'},
+	{"nufw-address", 1, NULL, 'L'},
+	{"client-address", 1, NULL, 'C'},
+	{"timeout", 1, NULL, 't'},
+
+	{0, 0, 0, 0}
+};
+
+
 /**
  * Display all command line options of NuAuth
  */
@@ -475,15 +513,16 @@ void print_usage()
 	fprintf(stdout,
 		"nuauth [-hDVv[v[v[v[v[v[v[v[v]]]]]]]]] [-l user_packet_port] [-C local_addr] [-L local_addr] \n"
 		"\t\t[-t packet_timeout]\n"
-		"\t-h : display this help and exit\n"
-		"\t-D : run as a daemon, send messages to syslog (else stdout/stderr)\n"
-		"\t-V : display version and exit\n"
-		"\t-v : increase verbosity level (+1 for each 'v') (max useful number : 10)\n"
-		"\t-p : specify listening TCP port (this port waits for nufw) (default : 4128)\n"
-		"\t-l : specify listening TCP port (this port waits for clients) (default : 4129)\n"
-		"\t-L : specify NUFW listening IP address (local) (this address waits for nufw data) (default : 127.0.0.1)\n"
-		"\t-C : specify clients listening IP address (local) (this address waits for clients auth) (default : 0.0.0.0)\n"
-		"\t-t : timeout to forget about packets when they don't match (default : 15 s)\n");
+		"\t-h (--help          ): display this help and exit\n"
+		"\t-c (--config        ): use alternate configuration file\n"
+		"\t-D (--daemon        ): run as a daemon, send debug messages to syslog (else stdout/stderr)\n"
+		"\t-V (--version       ): display version and exit\n"
+		"\t-v (--verbose       ): increase debug level (+1 for each 'v') (max useful number : 10)\n"
+		"\t-p (--nufw-port     ): specify listening TCP port (this port waits for nufw) (default : 4128)\n"
+		"\t-l (--client-port   ): specify listening TCP port (this port waits for clients) (default : 4129)\n"
+		"\t-L (--nufw-address  ): specify NUFW listening IP address (local) (this address waits for nufw data) (default : 127.0.0.1)\n"
+		"\t-C (--client-address): specify clients listening IP address (local) (this address waits for clients auth) (default : 0.0.0.0)\n"
+		"\t-t (--timeout       ): timeout to forget about packets when they don't match (default : 15 s)\n");
 }
 
 /**
@@ -491,13 +530,19 @@ void print_usage()
  */
 void parse_options(int argc, char **argv, command_line_params_t * params)
 {
-	char *options_list = "DhVvl:L:C:p:t:T:";
+	char *options_list = "DhVvc:l:L:C:p:t:T:";
 	int option;
 	int local_debug_level = 0;
 
 	/*parse options */
-	while ((option = getopt(argc, argv, options_list)) != -1) {
+	while ((option = getopt_long(argc, argv, options_list, long_options, NULL)) != -1) {
 		switch (option) {
+		case 'c':
+			/* configuration file */
+			g_free(params->configfile);
+			params->configfile = g_strdup(optarg);
+			break;
+
 		case 'V':
 			fprintf(stdout, "nuauth (version %s)\n",
 				NUAUTH_FULL_VERSION);
@@ -510,34 +555,26 @@ void parse_options(int argc, char **argv, command_line_params_t * params)
 
 		case 'l':
 			/* port we listen for auth answer */
-			g_free(nuauthconf->userpckt_port);
-			nuauthconf->userpckt_port = g_strdup(optarg);
-			printf("Waiting for user packets on TCP port %s\n",
-			       nuauthconf->userpckt_port);
+			g_free(params->userpckt_port);
+			params->userpckt_port = g_strdup(optarg);
 			break;
 
 		case 'p':
 			/* port we listen for auth answer */
-			g_free(nuauthconf->authreq_port);
-			nuauthconf->authreq_port = g_strdup(optarg);
-			printf("Waiting for nufw packets on TCP port %s\n",
-			       nuauthconf->authreq_port);
+			g_free(params->authreq_port);
+			params->authreq_port = g_strdup(optarg);
 			break;
 
 		case 'L':
 			/* Address we listen on for NUFW originating packets */
-			g_free(params->nuauth_nufw_listen_addr);
-			params->nuauth_nufw_listen_addr = g_strdup(optarg);
-			printf("Waiting for nufw daemon packets on %s\n",
-			       params->nuauth_nufw_listen_addr);
+			g_free(params->nufw_srv);
+			params->nufw_srv = g_strdup(optarg);
 			break;
 
 		case 'C':
 			/* Address we listen on for client originating packets */
-			g_free(params->nuauth_client_listen_addr);
-			params->nuauth_client_listen_addr = g_strdup(optarg);
-			printf("Waiting for clients auth packets on %s\n",
-			       params->nuauth_client_listen_addr);
+			g_free(params->client_srv);
+			params->client_srv = g_strdup(optarg);
 			break;
 
 		case 't':
@@ -556,7 +593,7 @@ void parse_options(int argc, char **argv, command_line_params_t * params)
 		}
 	}
 	if (local_debug_level) {
-		nuauthconf->debug_level = local_debug_level;
+		params->debug_level = local_debug_level;
 	}
 }
 
@@ -589,23 +626,23 @@ void nuauth_install_signals(gboolean sig_action)
 	/* intercept SIGTERM */
 	if (sigaction(SIGTERM, &action, &nuauthdatas->old_sigterm_hdl) !=
 	    0) {
-		g_message("Error modifying sigaction");
+		log_message(FATAL, DEBUG_AREA_MAIN, "Error modifying sigaction");
 		exit(EXIT_FAILURE);
 	}
 
 	/* intercept SIGINT */
 	if (sigaction(SIGINT, &action, &nuauthdatas->old_sigint_hdl) != 0) {
-		g_message("Error modifying sigaction");
+		log_message(FATAL, DEBUG_AREA_MAIN, "Error modifying sigaction");
 		exit(EXIT_FAILURE);
 	}
 
 	/* intercept SIGHUP */
 	memset(&action, 0, sizeof(action));
-	action.sa_handler = nuauth_reload;
+	action.sa_handler = (__sighandler_t) nuauth_reload;
 	sigemptyset(&(action.sa_mask));
 	action.sa_flags = 0;
 	if (sigaction(SIGHUP, &action, &nuauthdatas->old_sighup_hdl) != 0) {
-		g_message("Error modifying sigaction");
+		log_message(FATAL, DEBUG_AREA_MAIN, "Error modifying sigaction");
 		exit(EXIT_FAILURE);
 	}
 
@@ -619,44 +656,73 @@ void no_action_signals(int recv_signal)
 		case SIGINT:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 			    "[+] Nuauth received SIGINT (leaving)");
+		tls_common_deinit();
 		exit(EXIT_FAILURE);
 		break;
 		case SIGTERM:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 				"[+] Nuauth received SIGTERM (leaving)");
+		tls_common_deinit();
 		exit(EXIT_FAILURE);
 		break;
 		case SIGHUP:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 				"[+] Nuauth received SIGHUP (ignoring)");
+		tls_common_deinit();
 		break;
 	}
 }
 
+#define OVERWRITE_DATA(x) \
+	if (params->x) { \
+		g_free(nuauthconf->x); \
+		nuauthconf->x = g_strdup(params->x); \
+	}
+
+static void nuauthconf_from_cmdline(command_line_params_t *params)
+{
+	OVERWRITE_DATA(client_srv);
+	OVERWRITE_DATA(nufw_srv);
+	OVERWRITE_DATA(userpckt_port);
+	OVERWRITE_DATA(authreq_port);
+
+	nuauthconf->debug_level = params->debug_level;
+}
+
+#undef OVERWRITE_DATA
+
 /**
  * Configure NuAuth:
  *   - Init. glib threads: g_thread_init() ;
- *   - Read NuAuth configuration file: init_nuauthconf() ;
- *   - Init GNU TLS library: gnutls_global_init() ;
- *   - Create credentials: create_x509_credentials() ;
  *   - Read command line options: parse_options() ;
+ *   - Read NuAuth configuration file: init_nuauthconf() ;
  *   - Build configuration options: build_prenuauthconf() ;
+ *   - Init SSL library: nussl_init() ;
+ *   - Create credentials: create_x509_credentials() ;
  *   - Daemonize the process if asked: daemonize().
  */
 void configure_app(int argc, char **argv)
 {
 	command_line_params_t params;
-	int err;
-	struct rlimit core_limit;
+	int ret;
+	struct rlimit r_limit;
+	struct nuauth_params initconf;
+	struct nuauth_params *conf;
 
 	/* Avoid creation of core file which may contains username and password */
-	if (getrlimit(RLIMIT_CORE, &core_limit) == 0) {
+	if (getrlimit(RLIMIT_CORE, &r_limit) == 0) {
 #ifdef DEBUG_ENABLE
-		core_limit.rlim_cur = RLIM_INFINITY;
+		r_limit.rlim_cur = RLIM_INFINITY;
 #else
-		core_limit.rlim_cur = 0;
+		r_limit.rlim_cur = 0;
 #endif
-		setrlimit(RLIMIT_CORE, &core_limit);
+		setrlimit(RLIMIT_CORE, &r_limit);
+	}
+
+	if (getrlimit(RLIMIT_NOFILE, &r_limit) == 0) {
+		r_limit.rlim_cur = 8096;
+		r_limit.rlim_max = 8096;
+		setrlimit(RLIMIT_NOFILE, &r_limit);
 	}
 
 #ifndef DEBUG_ENABLE
@@ -681,37 +747,62 @@ void configure_app(int argc, char **argv)
 
 
 	/* set default parameters */
+	params.debug_level = 0;
 	params.daemonize = 0;
-	params.nuauth_client_listen_addr = NULL;
-	params.nuauth_nufw_listen_addr = NULL;
-
-	/* load configuration */
-	if (!init_nuauthconf(&nuauthconf)) {
-		fprintf(stderr,"Unable to load configuration\n");
-		exit(EXIT_FAILURE);
-	}
-
-	log_message(INFO, DEBUG_AREA_MAIN, "Starting NuFW authentication server.");
-
-	/* init gcrypt and gnutls */
-#ifdef GCRYPT_PTHEAD_IMPLEMENTATION
-	gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-#else
-	gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_gthread);
-#endif
-
-	err = gnutls_global_init();
-	if (err) {
-		fprintf(stderr,
-			"FATAL ERROR: gnutls global initialisation failed:\n"
-			"%s\n", gnutls_strerror(err));
-		exit(EXIT_FAILURE);
-	}
+	params.client_srv = NULL;
+	params.nufw_srv = NULL;
+	params.userpckt_port = NULL;
+	params.authreq_port = NULL;
+	params.configfile = NULL;
 
+	nuauthconf = &initconf;
+	/* check if configuration file is not given on command line */
 	parse_options(argc, argv, &params);
 
+	if (params.debug_level) {
+		nuauthconf->debug_level = params.debug_level;
+	} else {
+		nuauthconf->debug_level = DEFAULT_DEBUG_LEVEL;
+	}
+	nuauthconf->debug_areas = DEFAULT_DEBUG_AREAS;
+
+	if (params.configfile == NULL) {
+		nuauthconf->configfile = g_strdup(DEFAULT_CONF_FILE);
+	} else {
+		nuauthconf->configfile = g_strdup(params.configfile);
+	}
+
+	ret = nuauth_parse_configuration(nuauthconf->configfile);
+	if (ret < 0) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Cannot load configuration (file '%s')",
+			    nuauthconf->configfile);
+		exit(EXIT_FAILURE);
+	}
+
+	/* load configuration */
+	if (!init_nuauthconf(&conf)) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+				"Unable to load configuration");
+		exit(EXIT_FAILURE);
+	}
+
+	conf->configfile = nuauthconf->configfile;
+	nuauthconf = conf;
+	/* build configuration by overwritting config file option by
+	 * command line one */
+	nuauthconf_from_cmdline(&params);
+
 	build_prenuauthconf(nuauthconf, NULL, 0);
 
+	ret = nussl_init();
+	if ( ret != NUSSL_OK ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			"FATAL ERROR: NuSSL global initialisation failed.");
+		exit(EXIT_FAILURE);
+
+	}
+
 	if (nuauthconf->uses_utf8) {
 		setlocale(LC_ALL, "");
 	}
@@ -721,28 +812,50 @@ void configure_app(int argc, char **argv)
 		nuauthconf->debug_level = MAX_DEBUG_LEVEL;
 	if (nuauthconf->debug_level < MIN_DEBUG_LEVEL)
 		nuauthconf->debug_level = MIN_DEBUG_LEVEL;
-	log_message(INFO, DEBUG_AREA_MAIN, "debug_level is %i",
-		    nuauthconf->debug_level);
+	log_message(INFO, DEBUG_AREA_MAIN,
+			"Debug_level is %i", nuauthconf->debug_level);
 
+#if 0
 	/* init credential */
 	if (! create_x509_credentials()) {
-		fprintf(stderr,"Certificate initialization failed\n");
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
+				"Certificate initialization failed");
 		exit(EXIT_FAILURE);
 	}
+#endif
+	/* free command line structure */
+	g_free(params.client_srv);
+	g_free(params.nufw_srv);
+	g_free(params.userpckt_port);
+	g_free(params.authreq_port);
+	g_free(params.configfile);
 
+	debug_areas = DEFAULT_DEBUG_AREAS;
+	debug_level = nuauthconf->debug_level;
+
+	/* init gcrypt and gnutls */
 	if (params.daemonize == 1) {
 		daemonize();
+		set_glib_loghandlers(1 /* use only syslog */);
+
 	} else {
-		g_message("[+] Starting nuauth " VERSION
-			  " ($Revision: 5470 $) with config "
-			  DEFAULT_CONF_FILE );
-		g_message("Nuauth started in foreground (without -D option)"
-			  ", logging on stderr and stdout only (no syslog)");
+		set_glib_loghandlers(0 /* use syslog and stdout */);
+		log_message(FATAL, DEBUG_AREA_MAIN,
+				"[+] NuAuth ($Revision$) with config %s",
+				nuauthconf->configfile);
+		log_message(FATAL, DEBUG_AREA_MAIN,
+		"Nuauth started in foreground (without -D option), "
+		"logging on stderr, stdout and syslog");
 	}
+
+	/* declare default capabilities */
+	register_client_capa("HELLO", &(nuauthdatas->hello_capa));
+	register_client_capa("TCP", &(nuauthdatas->tcp_capa));
+	register_client_capa("UDP", &(nuauthdatas->udp_capa));
 }
 
 /**
- * Initialize all datas:
+ * Initialize all data:
  *   - Create different queues:
  *      - tls_push_queue: read in push_worker() ;
  *      - connections_queue: read in search_and_fill() ;
@@ -769,7 +882,7 @@ void configure_app(int argc, char **argv)
  * Other queue, threads, etc. are created elsewhere:
  *      - in tls_user_init(): tls_sasl_worker thread pool, tls_sasl_connect().
  */
-void init_nuauthdatas()
+void init_nuauthdata()
 {
 	block_thread_pools();
 	nuauthdatas->tls_push_queue = g_async_queue_new();
@@ -787,6 +900,8 @@ void init_nuauthdatas()
 	if (!nuauthdatas->connections_queue)
 		exit(EXIT_FAILURE);
 
+	init_protocol_extension(nuauthdatas);
+
 	/* init and load modules */
 	init_modules_system();
 	if (!load_modules())
@@ -828,12 +943,14 @@ void init_nuauthdatas()
 			      "localid", localid_auth);
 	}
 
+#ifdef BUILD_NUAUTH_COMMAND
 	if (nuauthconf->use_command_server) {
 		log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 			    "Creating command thread");
 		thread_new(&nuauthdatas->command_thread,
 			      "command", command_server);
 	}
+#endif
 
 	/* create thread for client request sender */
 	thread_new(&nuauthdatas->tls_pusher, "tls pusher", push_worker);
@@ -845,6 +962,8 @@ void init_nuauthdatas()
 	}
 
 	/* create TLS authentication server threads (auth + nufw) */
+	tls_common_init();
+
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 		    "Creating tls authentication server threads");
 	tls_user_start_servers(nuauthdatas->tls_auth_servers);
@@ -878,13 +997,18 @@ void main_cleanup()
 	if (DEBUG_OR_NOT(DEBUG_LEVEL_INFO, DEBUG_AREA_MAIN)) {
 		if (g_thread_pool_unprocessed(nuauthdatas->user_checkers)
 		    || g_thread_pool_unprocessed(nuauthdatas->
-						 acl_checkers)) {
+						 acl_checkers)
+		    || g_thread_pool_unprocessed(nuauthdatas->
+						 user_loggers)) {
 			g_message
-			    ("%u user/%u acl unassigned task(s), %d connection(s)",
+			    ("%u user/%u acl/%u log unassigned task(s), %d "
+			     "connection(s)",
 			     g_thread_pool_unprocessed(nuauthdatas->
 						       user_checkers),
 			     g_thread_pool_unprocessed(nuauthdatas->
 						       acl_checkers),
+			     g_thread_pool_unprocessed(nuauthdatas->
+						       user_loggers),
 			     g_hash_table_size(conn_list)
 			    );
 		}
@@ -928,11 +1052,12 @@ void nuauth_main_loop()
 	double sec;
 	unsigned long ms;
 
-	g_message("[+] NuFW authentication daemon started.");
+	log_message(FATAL, DEBUG_AREA_MAIN, "[+] NuAuth started.");
 
 	/* create timer and add main cleanup function to cleanup list */
 	timer = g_timer_new();
 	cleanup_func_push(main_cleanup);
+	cleanup_func_push(kill_expired_clients_session);
 
 	/* first sleep is one full second */
 	sleep.tv_sec = 1;
@@ -946,24 +1071,13 @@ void nuauth_main_loop()
 		/* a little sleep (one second) */
 		nanosleep(&sleep, NULL);
 
-		/* remove old connections */
-		g_timer_start(timer);
 		for (cleanup_it = cleanup_func_list;
 		     cleanup_it != NULL; cleanup_it = cleanup_it->next) {
 			cleanup_func_t cleanup = cleanup_it->data;
 			cleanup();
 		}
-		g_timer_stop(timer);
-
-		/* compute next sleep: one second minus (elasped time) */
-		sec = g_timer_elapsed(timer, &ms);
-		if (1 <= sec) {
-			sleep.tv_sec = 0;
-			sleep.tv_nsec = 0;
-		} else {
-			sleep.tv_sec = 0;
-			sleep.tv_nsec = (1000000 - ms) * 1000;
-		}
+		sleep.tv_sec = 1;
+		sleep.tv_nsec = 0;
 	}
 	g_timer_destroy(timer);
 }
@@ -972,14 +1086,14 @@ void nuauth_main_loop()
  * NuAuth entry point:
  *   - Configure application with: configure_app()
  *   - Install signals: nuauth_install_signals()
- *   - Init. all datas: init_nuauthdatas()
+ *   - Init. all data: init_nuauthdata()
  *   - Init. autdit: init_audit()
  *   - Run main loop: nuauth_main_loop()
  */
 int main(int argc, char *argv[])
 {
 	configure_app(argc, argv);
-	init_nuauthdatas();
+	init_nuauthdata();
 	nuauth_install_signals(TRUE);
 	init_audit();
 	nuauth_main_loop();
