@@ -5,11 +5,11 @@
  ** Changelog:
  **	IPv6 port by Victor Stinner
  **
- ** $Id: script.c 3770 2007-09-25 23:15:22Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +25,8 @@
 #include <string.h>
 #include <errno.h>
 
+#include <nubase.h>
+
 /**
  *
  * \ingroup LoggingNuauthModules
@@ -54,7 +56,7 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 	char *format;
 	gboolean ok;
 
-	FORMAT_IPV6(&c_session->addr, address);
+	format_ipv6(&c_session->addr, address, INET6_ADDRSTRLEN, NULL);
 	quoted_address = g_shell_quote(address);
 
 	if (state == SESSION_OPEN) {
@@ -65,7 +67,10 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 	ok = secure_snprintf(cmdbuffer, sizeof(cmdbuffer), format,
 			     quoted_username, quoted_address);
 	if (ok) {
-		system(cmdbuffer);
+		if (system(cmdbuffer) == -1) {
+			log_message(WARNING, DEBUG_AREA_MAIN,
+			    "Can't execute command (%s)!", cmdbuffer);
+		}
 	} else {
 		log_message(WARNING, DEBUG_AREA_MAIN,
 			    "Can't call script, command line truncated!");
@@ -84,7 +89,7 @@ G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_script module ($Revision: 3770 $)");
+		    "Log_script module ($Revision$)");
 	return TRUE;
 }
 
