@@ -1,13 +1,14 @@
 /*
- ** Copyright(C) 2004-2008 INL
+ ** Copyright(C) 2004-2009 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
+ **             Pierre Chifflier <chifflier@inl.fr>
  **
- ** $Id: tls_nufw.c 5478 2009-01-10 12:48:50Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +23,14 @@
 
 #include "auth_srv.h"
 
+#include <nubase.h>
+#include <nussl.h>
+
+#include <sys/socket.h>
+#include <sys/un.h>
+
+#include "nuauthconf.h"
+
 /**
  * \ingroup TLS
  * \defgroup TLSNufw TLS Nufw server
@@ -34,14 +43,22 @@
  * The main thread is tls_nufw_authsrv() which call tls_nufw_main_loop().
  */
 
+int nuauth_tls_max_servers = NUAUTH_TLS_MAX_SERVERS;
+int nufw_servers_connected = 0;
+
+extern struct nuauth_tls_t nuauth_tls;
+
 
 struct tls_nufw_context_t {
 	char *addr;
 	char *port;
 	int mx;
 	int sck_inet;
+	int sck_unix;
 	fd_set tls_rx_set;	/* read set */
 	GMutex *mutex;
+
+	nussl_session *server;
 };
 
 /**
@@ -66,15 +83,13 @@ static int treat_nufw_request(nufw_session_t * c_session)
 		return NU_EXIT_OK;
 
 	/* read data from nufw */
-	g_mutex_lock(c_session->tls_lock);
-	dgram_size =
-	    gnutls_record_recv(*(c_session->tls), dgram,
-			       CLASSIC_NUFW_PACKET_SIZE);
-	g_mutex_unlock(c_session->tls_lock);
+/*	g_mutex_lock(c_session->tls_lock); */
+	dgram_size = nussl_read(c_session->nufw_client, (char *)dgram, CLASSIC_NUFW_PACKET_SIZE);
+/*	g_mutex_unlock(c_session->tls_lock);*/
 	if (dgram_size < 0) {
 		log_message(INFO, DEBUG_AREA_GW,
 			    "nufw failure at %s:%d (%s)", __FILE__,
-			    __LINE__,gnutls_strerror(dgram_size));
+			    __LINE__, nussl_get_error(c_session->nufw_client));
 		return NU_EXIT_ERROR;
 	} else if (dgram_size == 0) {
 		log_message(INFO, DEBUG_AREA_GW,
@@ -94,9 +109,8 @@ static int treat_nufw_request(nufw_session_t * c_session)
 	}
 	/* decode data */
 	do {
-		ret =
-		    authpckt_decode(&dgram, (unsigned int *) &dgram_size,
-				    &current_conn);
+		ret = authpckt_decode(&dgram, (unsigned int *) &dgram_size,
+					&current_conn);
 		switch (ret) {
 		case NU_EXIT_ERROR:
 			return NU_EXIT_ERROR;
@@ -164,6 +178,21 @@ static int treat_nufw_request(nufw_session_t * c_session)
 }
 
 
+static int get_reverse_dns_info(struct sockaddr_storage *addr, char *buffer, size_t size)
+{
+	int ret;
+
+	ret = getnameinfo((const struct sockaddr*)addr,
+			sizeof(*addr),
+			buffer,
+			size,
+			NULL,
+			0,
+			0);
+
+	return ret;
+}
+
 
 /**
  * Function called on new NuFW connection: create a new TLS session using
@@ -177,70 +206,185 @@ int tls_nufw_accept(struct tls_nufw_context_t *context)
 	struct sockaddr_storage sockaddr;
 	struct sockaddr_in *sockaddr4 = (struct sockaddr_in *) &sockaddr;
 	struct sockaddr_in6 *sockaddr6 = (struct sockaddr_in6 *) &sockaddr;
-	struct in6_addr addr;
-	char addr_ascii[INET6_ADDRSTRLEN];
-	unsigned int len_inet;
+	char address[INET6_ADDRSTRLEN];
+	int ret;
+	char peername[256];
+	int port;
+	socklen_t len_inet = sizeof(sockaddr);
+	char cipher[256];
+
 	nufw_session_t *nu_session;
 
-	/* Accept the connection */
-	len_inet = sizeof sockaddr;
-	conn_fd = accept(context->sck_inet,
-			 (struct sockaddr *) &sockaddr, &len_inet);
-	if (conn_fd == -1) {
-		log_message(WARNING, DEBUG_AREA_GW, "accept");
+	/* initialize TLS */
+	nu_session = g_new0(nufw_session_t, 1);
+
+	nu_session->connect_timestamp = time(NULL);
+	nu_session->usage = 0;
+	nu_session->alive = TRUE;
+
+	/* We have to wait the first packet */
+	nu_session->proto_version = PROTO_UNKNOWN;
+
+	nu_session->nufw_client = nussl_session_accept(context->server);
+	if ( ! nu_session->nufw_client ) {
+		g_free(nu_session);
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"Unable to allocate nufw server connection : %s",
+				nussl_get_error(context->server));
+		return 1;
+	}
+
+	/* Check number of connected servers */
+	if ( nufw_servers_connected >= nuauth_tls_max_servers ) {
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"too many servers (%d configured)",
+				nuauth_tls_max_servers);
+		nussl_session_destroy(nu_session->nufw_client);
+		g_free(nu_session);
+		return 1;
+	}
+
+	if (nussl_session_getpeer(nu_session->nufw_client, (struct sockaddr *) &sockaddr, &len_inet) != NUSSL_OK)
+	{
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"Unable to get peername of NuFW dameon : %s",
+				nussl_get_error(nu_session->nufw_client));
+		nussl_session_destroy(nu_session->nufw_client);
+		g_free(nu_session);
+		return 1;
 	}
 
 	/* Extract client address (convert it to IPv6 if it's IPv4) */
 	if (sockaddr6->sin6_family == AF_INET) {
-		ipv4_to_ipv6(sockaddr4->sin_addr, &addr);
+		ipv4_to_ipv6(sockaddr4->sin_addr, &nu_session->peername);
+		port = ntohs(sockaddr4->sin_port);
 	} else {
-		addr = sockaddr6->sin6_addr;
+		nu_session->peername = sockaddr6->sin6_addr;
+		port = ntohs(sockaddr6->sin6_port);
 	}
 
-	/* test if server is in the list of authorized servers */
-	if (!check_inaddr_in_array(&addr, nuauthconf->authorized_servers)) {
-		FORMAT_IPV6(&addr, addr_ascii);
-		log_message(WARNING, DEBUG_AREA_GW,
-				"unwanted nufw server (%s)", addr_ascii);
-		close(conn_fd);
+	format_ipv6(&nu_session->peername, address, sizeof(address), NULL);
+	log_message(DEBUG, DEBUG_AREA_MAIN,
+			"nuauth: nufw connection attempt from %s\n",
+			address);
+
+	/* get canonical (first) name and set it in ssl session, so that
+	 * we can verify if peer name matches certificate CN entry
+	 */
+	ret = get_reverse_dns_info(&sockaddr, peername, sizeof(peername));
+	nussl_set_hostinfo(nu_session->nufw_client, peername, port);
+
+	/* copy verification flag from server session */
+	nussl_set_session_flag(nu_session->nufw_client,
+		NUSSL_SESSFLAG_IGNORE_ID_MISMATCH,
+		nussl_get_session_flag(context->server, NUSSL_SESSFLAG_IGNORE_ID_MISMATCH)
+		);
+
+	// XXX default value is 30s, should be a configuration value
+	nussl_set_connect_timeout(nu_session->nufw_client, 30);
+
+	ret = nussl_session_handshake(nu_session->nufw_client, context->server);
+	if ( ret ) {
+		log_message(WARNING, DEBUG_AREA_MAIN,
+				"Error during TLS handshake with nufw server %s : %s",
+				address,
+				nussl_get_error(context->server));
+		nussl_session_destroy(nu_session->nufw_client);
+		g_free(nu_session);
 		return 1;
 	}
-#if 0
-	if (conn_fd >= nuauth_tls_max_servers) {
-		log_message(WARNING, DEBUG_AREA_GW,
-			    "too much servers (%d configured)",
-			    nuauth_tls_max_servers);
-		close(conn_fd);
-		continue;
+
+	cipher[0] = '\0';
+	nussl_session_get_cipher(nu_session->nufw_client, cipher, sizeof(cipher));
+	log_message(INFO, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+		    "TLS handshake with nufw server %s succeeded, cipher is %s",
+		    address, cipher);
+
+	/* Check certificate hook */
+	ret = modules_check_certificate(nu_session->nufw_client);
+	if ( ret ) {
+		log_message(WARNING, DEBUG_AREA_MAIN,
+				"New client connection from %s failed during modules_check_certificate()",
+				address);
+		nussl_session_destroy(nu_session->nufw_client);
+		g_free(nu_session);
+		return 1;
 	}
-#endif
+
+	nufw_servers_connected++;
+
+	conn_fd = nussl_session_get_fd(nu_session->nufw_client);
+
+	nu_session->tls_lock = g_mutex_new();
+	add_nufw_server(conn_fd, nu_session);
+	FD_SET(conn_fd, &context->tls_rx_set);
+	if (conn_fd + 1 > context->mx)
+		context->mx = conn_fd + 1;
+	g_message("[+] NuFW: new NuFW server (%s) connected on socket %d",
+		  address, conn_fd);
+
+	return 0;
+}
+
+int tls_nufw_accept_unix(struct tls_nufw_context_t *context)
+{
+	int conn_fd;
+	struct sockaddr_un sockaddr;
+	socklen_t len_unix = sizeof(sockaddr);
+
+	nufw_session_t *nu_session;
 
 	/* initialize TLS */
 	nu_session = g_new0(nufw_session_t, 1);
+
 	nu_session->connect_timestamp = time(NULL);
 	nu_session->usage = 0;
 	nu_session->alive = TRUE;
-	nu_session->peername = addr;
-	nu_session->socket = conn_fd;
+
 	/* We have to wait the first packet */
 	nu_session->proto_version = PROTO_UNKNOWN;
-	if (tls_connect(conn_fd, &(nu_session->tls)) == SASL_OK
-	    && tls_check_fqdn(&sockaddr, &addr, nu_session->tls) == SASL_OK) {
-		nu_session->tls_lock = g_mutex_new();
-		add_nufw_server(conn_fd, nu_session);
-		FD_SET(conn_fd, &context->tls_rx_set);
-		if (conn_fd + 1 > context->mx)
-			context->mx = conn_fd + 1;
-		FORMAT_IPV6(&addr, addr_ascii);
-		g_message("[+] new NuFW server (%s) connected on socket %d",
-			  addr_ascii, conn_fd);
-	} else {
-		FORMAT_IPV6(&addr, addr_ascii);
-		log_message(WARNING, DEBUG_AREA_GW,
-				"TLS handshake with nufw server failed (%s)", addr_ascii);
+
+	nu_session->nufw_client = NULL;
+
+	conn_fd = accept(context->sck_unix, (struct sockaddr*)&sockaddr, &len_unix);
+	if ( conn_fd < 0 ) {
 		g_free(nu_session);
-		close(conn_fd);
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"Error while accepting nufw server connection");
+		return 1;
 	}
+
+	/* Check number of connected servers */
+	if ( nufw_servers_connected >= nuauth_tls_max_servers ) {
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"too many servers (%d configured)",
+				nuauth_tls_max_servers);
+		close(conn_fd);
+		g_free(nu_session);
+		return 1;
+	}
+
+	nu_session->nufw_client = nussl_session_create_with_fd(conn_fd, 0 /* verify */);
+	if ( ! nu_session->nufw_client ) {
+		close(conn_fd);
+		g_free(nu_session);
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"Unable to allocate nufw server connection : %s",
+				nussl_get_error(context->server));
+		return 1;
+	}
+
+
+	nufw_servers_connected++;
+
+	nu_session->tls_lock = g_mutex_new();
+	add_nufw_server(conn_fd, nu_session);
+	FD_SET(conn_fd, &context->tls_rx_set);
+	if (conn_fd + 1 > context->mx)
+		context->mx = conn_fd + 1;
+	g_message("[+] NuFW: new NuFW server connected on unix socket %d",
+		  conn_fd);
+
 	return 0;
 }
 
@@ -256,7 +400,9 @@ void tls_nufw_main_loop(struct tls_nufw_context_t *context, GMutex * mutex)
 	int n, c, z;
 	fd_set wk_set;		/* working set */
 	struct timeval tv;
+	char *unix_path;
 
+	unix_path = nuauth_config_table_get("nuauth_client_listen_socket");
 	log_message(INFO, DEBUG_AREA_GW,
 		    "[+] NuAuth is waiting for NuFW connections.");
 	while (g_mutex_trylock(mutex)) {
@@ -289,16 +435,9 @@ void tls_nufw_main_loop(struct tls_nufw_context_t *context, GMutex * mutex)
 					struct stat s;
 					if (FD_ISSET(i, &context->tls_rx_set)){
 						if (fstat(i, &s)<0) {
-							nufw_session_t *c_session =
-								acquire_nufw_session_by_socket(i);
 							log_message(CRITICAL, DEBUG_AREA_USER,
 									"Warning: %d is a bad file descriptor.", i);
 							FD_CLR(i, &context->tls_rx_set);
-							FD_CLR(c, &wk_set);
-							if (c_session) {
-								c_session->alive = FALSE;
-								declare_dead_nufw_session(c_session);
-							}
 						}
 					}
 				}
@@ -310,25 +449,7 @@ void tls_nufw_main_loop(struct tls_nufw_context_t *context, GMutex * mutex)
 				    __FILE__, __LINE__, g_strerror(errno));
 			nuauth_ask_exit();
 			break;
-		}
-
-		/* if tls params are invalid we disconnect all nufw servers */
-		if (nuauth_tls.invalid_tls_params) {
-			for (c = 0; c < context->mx; ++c) {
-				if (c == context->sck_inet)
-					continue;
-
-				if (FD_ISSET(c, &context->tls_rx_set)) {
-					nufw_session_t *c_session =
-						acquire_nufw_session_by_socket(c);
-					FD_CLR(c, &context->tls_rx_set);
-					FD_CLR(c, &wk_set);
-					declare_dead_nufw_session(c_session);
-				}
-			}
-		}
-
-		if (!n) {
+		} else if (!n) {
 			continue;
 		}
 
@@ -339,35 +460,41 @@ void tls_nufw_main_loop(struct tls_nufw_context_t *context, GMutex * mutex)
 			}
 		}
 
+		/* Check if a connect has occured */
+		if (context->sck_unix > 0 && FD_ISSET(context->sck_unix, &wk_set)) {
+			if (tls_nufw_accept_unix(context)) {
+				continue;
+			}
+		}
+
 		/* check for server activity */
 		for (c = 0; c < context->mx; ++c) {
 			if (c == context->sck_inet)
 				continue;
+			if (c == context->sck_unix)
+				continue;
 
 			if (FD_ISSET(c, &wk_set)) {
 				nufw_session_t *c_session;
 				debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW,
 						  "nufw activity on socket %d",
 						  c);
-				c_session =
-					acquire_nufw_session_by_socket(c);
-				if (c_session == NULL) {
-					log_message(WARNING, DEBUG_AREA_GW,
-							  "nufw server not found for %d",
-							  c);
+
+				c_session = acquire_nufw_session_by_socket(c);
+				if (! c_session) {
+					FD_CLR(c, &context->tls_rx_set);
 					continue;
 				}
-				if (treat_nufw_request(c_session) ==
-				    NU_EXIT_ERROR) {
+				if (treat_nufw_request(c_session) == NU_EXIT_ERROR) {
 					/* get session link with c */
 					debug_log_message(DEBUG, DEBUG_AREA_GW,
 							  "nufw server disconnect on %d",
 							  c);
 					FD_CLR(c, &context->tls_rx_set);
 					declare_dead_nufw_session(c_session);
-				}
-				else
+				} else {
 					release_nufw_session(c_session);
+				}
 			}
 		}
 
@@ -378,15 +505,27 @@ void tls_nufw_main_loop(struct tls_nufw_context_t *context, GMutex * mutex)
 		}
 	}
 	close(context->sck_inet);
+	close(context->sck_unix);
+	if (unix_path)
+		unlink(unix_path);
 }
 
 /**
  * Initialize the NuFW TLS servers thread
+ * 0 if error, 1 on success
  */
 int tls_nufw_init(struct tls_nufw_context_t *context)
 {
 	int socket_fd;
+	int unix_socket_fd;
 	char *errmsg;
+	char *unix_path;
+
+/* config init */
+	int ret;
+	int int_requestcert;
+	int int_disable_fqdn_check;
+	char *dh_params_file;
 
 	context->sck_inet = nuauth_bind(&errmsg, context->addr, context->port, "nufw");
 	if (context->sck_inet < 0) {
@@ -396,10 +535,25 @@ int tls_nufw_init(struct tls_nufw_context_t *context)
 			    "Check that nuauth is not running twice. Exiting nuauth!");
 		return 0;
 	}
-#if 0
+
+	unix_path = nuauth_config_table_get("nuauth_client_listen_socket");
+	if (unix_path) {
+		context->sck_unix = nuauth_bind_unix(&errmsg, unix_path);
+		if (context->sck_unix < 0) {
+			log_message(FATAL, DEBUG_AREA_GW | DEBUG_AREA_MAIN,
+					"FATAL ERROR: NuFW unix bind error: %s", errmsg);
+			log_message(FATAL, DEBUG_AREA_GW | DEBUG_AREA_MAIN,
+					"Check that nuauth is not running twice. Exiting nuauth!");
+			return 0;
+		}
+	} else {
+		context->sck_unix = -1;
+	}
+
+#if 0 /* XXX: Already commented in 2.2 */
 	struct sigaction action;
 
-	char *configfile = DEFAULT_CONF_FILE;
+	char *configfile = nuauthconf->configfile;
 	gpointer vpointer;
 	confparams_t nuauth_tls_vars[] = {
 		{"nuauth_tls_max_servers", G_TOKEN_INT,
@@ -410,8 +564,7 @@ int tls_nufw_init(struct tls_nufw_context_t *context)
 	/* parse conf file */
 	if (!parse_conffile(configfile,
 		       sizeof(nuauth_tls_vars) / sizeof(confparams_t),
-		       nuauth_tls_vars))
-	{
+		       nuauth_tls_vars)) {
 	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
 		return 0;
 	}
@@ -428,17 +581,101 @@ int tls_nufw_init(struct tls_nufw_context_t *context)
 	/* Listen ! */
 	socket_fd = listen(context->sck_inet, 20);
 	if (socket_fd == -1) {
-		g_warning("nufw listen() failed, exiting");
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "nufw listen() failed, exiting");
 		exit(EXIT_FAILURE);
-		return 0;
+	}
+
+	if (context->sck_unix >= 0) {
+		unix_socket_fd = listen(context->sck_unix, 20);
+		if (unix_socket_fd == -1) {
+			log_message(FATAL, DEBUG_AREA_MAIN,
+					"nufw unix_socket listen() failed, exiting");
+			exit(EXIT_FAILURE);
+		}
 	}
 
 
 	/* init fd_set */
 	context->mx = context->sck_inet + 1;
+	if (context->sck_unix > context->sck_inet)
+		context->mx = context->sck_unix + 1;
 
 	FD_ZERO(&context->tls_rx_set);
 	FD_SET(context->sck_inet, &context->tls_rx_set);
+	if (context->sck_unix >=0)
+		FD_SET(context->sck_unix, &context->tls_rx_set);
+
+	/* TODO: read values specific to nufw connection */
+	nuauth_tls_max_servers = nuauth_config_table_get_or_default_int("nuauth_tls_max_servers", NUAUTH_TLS_MAX_SERVERS);
+	int_requestcert = nuauth_config_table_get_or_default_int("nuauth_tls_request_cert", FALSE);
+	dh_params_file = nuauth_config_table_get("nuauth_tls_dh_params");
+	/* {"nuauth_tls_auth_by_cert", G_TOKEN_INT, FALSE, NULL}, */
+
+	int_disable_fqdn_check = nuauth_config_table_get_or_default_int("nuauth_tls_disable_nufw_fqdn_check", FALSE);
+
+	/* TODO: use a nufw specific value of request_cert */
+	context->server = nussl_session_create_with_fd(context->sck_inet, nuauth_tls.request_cert);
+	if ( ! context->server ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Cannot create NuSSL session!");
+		exit(EXIT_FAILURE);
+	}
+
+	ret = NUSSL_ERROR;
+	if (dh_params_file) {
+		ret = nussl_session_set_dh_file(context->server, dh_params_file);
+	}
+	if (ret != NUSSL_OK &&
+		nussl_session_set_dh_bits(context->server, DH_BITS) != NUSSL_OK) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Unable to initialize Diffie Hellman params.");
+		exit(EXIT_FAILURE);
+	}
+
+	ret = nussl_ssl_set_keypair(context->server, nuauth_tls.cert, nuauth_tls.key);
+	if ( ret != NUSSL_OK ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Failed to load nufw key/certificate: %s",
+			    nussl_get_error(context->server));
+		exit(EXIT_FAILURE);
+	}
+
+	ret = nussl_ssl_trust_cert_file(context->server, nuauth_tls.ca);
+	if ( ret != NUSSL_OK ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Failed to load nufw certificate authority (nuauth_tls_cacert): %s",
+			    nussl_get_error(context->server));
+		exit(EXIT_FAILURE);
+	}
+
+	if (nuauth_tls.capath) {
+		ret = nussl_ssl_trust_dir(context->server, nuauth_tls.capath);
+		if ( ret != NUSSL_OK ) {
+			log_message(FATAL, DEBUG_AREA_MAIN,
+					"Failed to load user certificate authority directory: %s",
+					nussl_get_error(context->server));
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (nuauth_tls.crl_file) {
+		ret = nussl_ssl_set_crl_file(context->server, nuauth_tls.crl_file, nuauth_tls.ca);
+		if ( ret != NUSSL_OK ) {
+			log_message(FATAL, DEBUG_AREA_MAIN,
+					"Failed to load certificate revocation list (CRL): %s",
+					nussl_get_error(context->server));
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (nuauth_tls.ciphers) {
+		nussl_session_set_ciphers(context->server, nuauth_tls.ciphers);
+	}
+
+	if (int_disable_fqdn_check)
+		nussl_set_session_flag(context->server, NUSSL_SESSFLAG_IGNORE_ID_MISMATCH, 1);
+
 	return 1;
 }
 
@@ -493,6 +730,42 @@ void tls_nufw_start_servers(GSList *servers)
 }
 
 
+/**
+ * Refresh crl in the nufw contexts
+ *
+ */
+
+void tls_crl_update_nufw_session(GSList *session)
+{
+
+	GSList *listrunner = session;
+	int ret;
+
+	while ( listrunner ) {
+		struct nuauth_thread_t *nuauth_thread = listrunner->data;
+		struct tls_nufw_context_t *context = nuauth_thread->data;
+
+		// Don't update the CRL when nufw is not yet connected
+		if (context->server == NULL) {
+			listrunner = g_slist_next(listrunner);
+			continue;
+		}
+
+		ret = nussl_ssl_set_crl_file(context->server, nuauth_tls.crl_file, nuauth_tls.ca);
+
+		if (ret != NUSSL_OK) {
+			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
+					"[%i] NuFW TLS: CRL file reloading failed (%s)",
+					getpid(), nussl_get_error(context->server));
+		}
+
+		listrunner = g_slist_next(listrunner);
+
+	}
+	g_slist_free(listrunner);
+
+}
+
 
 /**
  * @}
